""" Custom admin site for the Saved Query
"""
from django.contrib import admin


class CustomSavedQueryAdmin(admin.ModelAdmin):
    """CustomSavedQueryAdmin"""

    readonly_fields = ["template", "query", "displayed_query"]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Saved Queries"""
        return False



"""Url router for the administration site
"""
from django.contrib import admin

from core_main_app.admin import core_admin_site
from core_explore_example_app.components.explore_data_structure.models import (
    ExploreDataStructure,
)
from core_explore_example_app.components.persistent_query_example.admin_site import (
    CustomPersistentQueryExampleAdmin,
)
from core_explore_example_app.components.persistent_query_example.models import (
    PersistentQueryExample,
)
from core_explore_example_app.components.saved_query.admin_site import (
    CustomSavedQueryAdmin,
)
from core_explore_example_app.components.explore_data_structure.admin_site import (
    CustomExploreDataStructureAdmin,
)
from core_explore_example_app.components.saved_query.models import SavedQuery

admin_urls = []


admin.site.register(ExploreDataStructure, CustomExploreDataStructureAdmin)
admin.site.register(PersistentQueryExample, CustomPersistentQueryExampleAdmin)
admin.site.register(SavedQuery, CustomSavedQueryAdmin)
urls = core_admin_site.get_urls()
core_admin_site.get_urls = lambda: admin_urls + urls



"""Utils for the query builder
"""
from os.path import join

from django.template import loader

from core_main_app.settings import MONGODB_INDEXING
from xml_utils.xsd_types.xsd_types import get_xsd_numbers, get_xsd_gregorian_types
from core_explore_example_app.utils.xml import get_enumerations


class BranchInfo:
    """Store information about a branch from the xml schema while it is being processed for field selection"""

    def __init__(self, keep_the_branch=False, selected_leaves=None):
        self.keep_the_branch = keep_the_branch
        self.selected_leaves = selected_leaves if selected_leaves is not None else []

    def add_selected_leaf(self, leaf_id):
        """add_selected_leaf
        Args:
            leaf_id:

        Returns
        """
        self.selected_leaves.append(leaf_id)
        self.keep_the_branch = True


# Util functions


def prune_html_tree(html_tree):
    """Create a custom HTML tree from fields chosen by the user

    Args:
        html_tree:

    Returns:

    """
    any_branch_checked = False

    list_ul = html_tree.findall("./ul")
    for ul in list_ul:
        branch_info = prune_ul(ul)
        if branch_info.keep_the_branch:
            any_branch_checked = True

    return any_branch_checked


def prune_ul(ul):
    """Process the ul element of an HTML list

    Args:
        ul:

    Returns:

    """
    list_li = ul.findall("./li")
    branch_info = BranchInfo()

    for li in list_li:
        li_branch_info = prune_li(li)
        if li_branch_info.keep_the_branch:
            branch_info.keep_the_branch = True
        branch_info.selected_leaves.extend(li_branch_info.selected_leaves)

    checkbox = ul.find("./input[@type='checkbox']")
    if checkbox is not None:
        if "value" in checkbox.attrib and checkbox.attrib["value"] == "true":
            # set element class
            parent_li = ul.getparent()
            element_id = parent_li.attrib["class"]
            add_selection_attributes(parent_li, "element", element_id)
            # tells to keep this branch until this leaf
            branch_info.add_selected_leaf(element_id)

    if not branch_info.keep_the_branch:
        add_selection_attributes(ul, "none")

    return branch_info


def prune_li(li):
    """Process the li element of an HTML list

    Args:
        li:

    Returns:

    """
    list_ul = li.findall("./ul")
    branch_info = BranchInfo()

    if len(list_ul) != 0:
        selected_leaves = []
        for ul in list_ul:
            ul_branch_info = prune_ul(ul)
            if ul_branch_info.keep_the_branch:
                branch_info.keep_the_branch = True
            selected_leaves.extend(ul_branch_info.selected_leaves)

        # sub element queries available when more than one selected elements under the same element,
        # and data stored in MongoDB
        if MONGODB_INDEXING and len(selected_leaves) > 1:
            # not for the choices
            if li[0].tag != "select":
                # TODO: check if test useful
                if "select_class" not in li.attrib:
                    leaves_id = " ".join(selected_leaves)
                    add_selection_attributes(li, "parent", leaves_id)
        if not branch_info.keep_the_branch:
            add_selection_attributes(li, "none")

        return branch_info
    else:
        try:
            checkbox = li.find("./input[@type='checkbox']")
            if checkbox.attrib["value"] == "false":
                add_selection_attributes(li, "none")
                return branch_info
            else:
                element_id = li.attrib["class"]
                add_selection_attributes(li, "element", element_id)
                # tells to keep this branch until this leaf
                branch_info.add_selected_leaf(element_id)
                return branch_info
        except:
            return branch_info


def add_selection_attributes(element, select_class, select_id=None):
    """Add css attribute to selected element

    Args:
        element:
        select_class:
        select_id:

    Returns:

    """
    element.attrib["select_class"] = select_class
    if select_id is not None:
        element.attrib["select_id"] = select_id


# Rendering functions


def render_yes_or_not():
    """Return a string that represents an html select with yes or not options

    Returns:

    """
    return _render_template(
        join("core_explore_example_app", "user", "query_builder", "yes_no.html")
    )


def render_and_or_not():
    """Return a string that represents an html select with AND, OR, NOT options

    Returns:

    """
    return _render_template(
        join("core_explore_example_app", "user", "query_builder", "and_or_not.html")
    )


def render_numeric_select():
    """Return a string that represents an html select with numeric comparisons

    Returns:

    """
    return _render_template(
        join("core_explore_example_app", "user", "query_builder", "numeric_select.html")
    )


def render_value_input():
    """Return an input to type a value

    Returns:

    """
    return _render_template(
        join("core_explore_example_app", "user", "query_builder", "input.html")
    )


def render_gregorian_strict_match():
    """Return an input to type a value

    Returns:

    """
    return _render_template(
        join(
            "core_explore_example_app",
            "user",
            "query_builder",
            "gregorian_strict_match.html",
        )
    )


def render_string_select():
    """Return an input to type a value

    Returns:

    """
    return _render_template(
        join("core_explore_example_app", "user", "query_builder", "string_select.html")
    )


def render_initial_form():
    """Render the initial Query Builder

    Returns:

    """
    return _render_template(
        join("core_explore_example_app", "user", "query_builder", "initial_form.html")
    )


def render_remove_button():
    """Return html of a remove button

    Returns:

    """
    return _render_template(
        join("core_explore_example_app", "user", "query_builder", "remove.html")
    )


def render_add_button():
    """Return html of an add button

    Returns:

    """
    return _render_template(
        join("core_explore_example_app", "user", "query_builder", "add.html")
    )


def render_enum(enums):
    """Return html select from an enumeration

    Args:
        enums:

    Returns:

    """
    context = {
        "enums": enums,
    }
    return _render_template(
        join("core_explore_example_app", "user", "query_builder", "enum.html"), context
    )


def render_new_query(tag_id, query, is_first=False):
    """Return an html string for a new query

    Args:
        tag_id:
        query:
        is_first:

    Returns:

    """
    context = {"tagID": tag_id, "query": query, "first": is_first}
    return _render_template(
        join("core_explore_example_app", "user", "query_builder", "new_query.html"),
        context,
    )


def render_new_criteria(tag_id):
    """Return an html string for a new query

    Args:
        tag_id:

    Returns:

    """
    context = {
        "tagID": tag_id,
    }
    return _render_template(
        join("core_explore_example_app", "user", "query_builder", "new_criteria.html"),
        context,
    )


def render_sub_elements_query(parent_name, form_fields):
    """Return an html string for a query on sub-elements

    Args:


    Returns:

    """
    context = {
        "parent_name": parent_name,
        "form_fields": form_fields,
    }
    return _render_template(
        join(
            "core_explore_example_app",
            "user",
            "query_builder",
            "sub_elements_query.html",
        ),
        context,
    )


def get_element_value(element_field):
    """Get value from field

    Args:
        element_field:

    Returns:

    """
    return element_field["value"] if "value" in element_field else None


def get_element_comparison(element_field):
    """Get comparison operator from field

    Args:
        element_field:

    Returns:

    """
    return element_field["comparison"] if "comparison" in element_field else "is"


def get_user_inputs(element_type, data_structure_element, default_prefix):
    """Get user inputs from element type

    Args:
        element_type:
        data_structure_element:
        default_prefix:

    Returns:

    """
    try:
        if element_type is not None and element_type.startswith(
            "{0}:".format(default_prefix)
        ):
            # numeric
            if element_type in get_xsd_numbers(default_prefix):
                user_inputs = render_numeric_select() + render_value_input()
            # gregorian date
            elif element_type in get_xsd_gregorian_types(default_prefix):
                user_inputs = render_gregorian_strict_match() + render_value_input()
            # string
            else:
                user_inputs = render_string_select() + render_value_input()
        else:
            # enumeration
            enums = get_enumerations(data_structure_element)
            user_inputs = render_enum(enums)
    except:
        # default renders string form
        user_inputs = render_string_select() + render_value_input()

    return user_inputs


def _render_template(template_path, context=None):
    """Return an HTML string rendered from the template

    Args:
        template_path:

    Returns:

    """
    if context is None:
        context = {}
    template = loader.get_template(template_path)
    return template.render(context)



""" Authentication tests for Saved Query REST API
"""
from django.test import SimpleTestCase
from mock.mock import patch
from rest_framework import status

from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
import core_explore_example_app.components.saved_query.api as saved_query_api
from core_explore_example_app.components.saved_query.models import SavedQuery
from core_explore_example_app.rest.saved_query.serializers import SavedQuerySerializer
from core_explore_example_app.rest.saved_query.views import (
    SavedQueryList,
    SavedQueryDetail,
)


class TestSavedQueryListGetPermissions(SimpleTestCase):
    """Test Saved Query List Get Permissions"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_get(SavedQueryList.as_view(), None)

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(saved_query_api, "get_all")
    def test_authenticated_returns_http_200(self, mock_saved_query_get_all):
        """test_authenticated_returns_http_200"""

        mock_saved_query_get_all.return_value = {}
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(SavedQueryList.as_view(), mock_user)

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(saved_query_api, "get_all")
    def test_staff_returns_http_200(self, mock_saved_query_get_all):
        """test_staff_returns_http_200"""

        mock_saved_query_get_all.return_value = {}
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(SavedQueryList.as_view(), mock_user)

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestSavedQueryDetailGetPermissions(SimpleTestCase):
    """Test Saved Query Detail Get Permissions"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_get(
            SavedQueryDetail.as_view(), None, param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(saved_query_api, "get_by_id")
    @patch.object(SavedQuerySerializer, "data")
    def test_authenticated_returns_http_200(
        self, mock_saved_query_serializer, mock_saved_query_get_by_id
    ):
        """test_authenticated_returns_http_200"""

        mock_saved_query_get_by_id.return_value = None
        mock_saved_query_serializer.return_value = {}
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            SavedQueryDetail.as_view(), mock_user, param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(saved_query_api, "get_by_id")
    @patch.object(SavedQuerySerializer, "data")
    def test_staff_returns_http_200(
        self, mock_saved_query_serializer, mock_saved_query_get_by_id
    ):
        """test_staff_returns_http_200"""

        mock_saved_query_get_by_id.return_value = None
        mock_saved_query_serializer.return_value = {}
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            SavedQueryDetail.as_view(), mock_user, param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestSavedQueryDetailDeletePermissions(SimpleTestCase):
    """Test Saved Query Detail Delete Permissions"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_delete(
            SavedQueryDetail.as_view(), None, param={"pk": "0"}
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(saved_query_api, "get_by_id")
    def test_not_owner_returns_http_403(self, mock_saved_query_get_by_id):
        """test_not_owner_returns_http_403"""

        mock_saved_query_get_by_id.return_value = SavedQuery(user_id="1")
        mock_user = create_mock_user("2")

        response = RequestMock.do_request_delete(
            SavedQueryDetail.as_view(), mock_user, param={"pk": "0"}, data=None
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(saved_query_api, "get_by_id")
    @patch.object(saved_query_api, "delete")
    def test_owner_returns_http_204(
        self, mock_saved_query_delete, mock_saved_query_get_by_id
    ):
        """test_owner_returns_http_204"""

        mock_saved_query_get_by_id.return_value = SavedQuery(user_id="1")
        mock_saved_query_delete.return_value = None
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_delete(
            SavedQueryDetail.as_view(), mock_user, param={"pk": "0"}, data=None
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(saved_query_api, "get_by_id")
    @patch.object(saved_query_api, "delete")
    def test_staff_returns_http_204(
        self, mock_saved_query_delete, mock_saved_query_get_by_id
    ):
        """test_staff_returns_http_204"""

        mock_saved_query_get_by_id.return_value = SavedQuery(user_id="1")
        mock_saved_query_delete.return_value = None
        mock_user = create_mock_user("5", is_staff=True)

        response = RequestMock.do_request_delete(
            SavedQueryDetail.as_view(), mock_user, param={"pk": "0"}, data=None
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)



"""Util to build queries for mongo db
"""
import json
import re
from typing import List, Any

from core_main_app.commons.exceptions import DoesNotExist
from core_main_app.components.template import api as template_api
from core_main_app.utils.xml import xpath_to_dot_notation
from core_parser_app.components.data_structure_element import (
    api as data_structure_element_api,
)
from xml_utils.xsd_tree.operations.namespaces import get_namespaces, get_default_prefix
from xml_utils.xsd_types.xsd_types import (
    get_xsd_numbers,
    get_xsd_floating_numbers,
    get_xsd_gregorian_types,
)
from core_explore_example_app.commons.exceptions import MongoQueryException
from core_explore_example_app.components.saved_query import api as saved_query_api
from core_explore_example_app.utils.query_builder import (
    get_element_value,
    get_element_comparison,
)
from core_explore_example_app.utils.xml import validate_element_value


def build_query_criteria(query, is_not=False):
    """Builds a criteria for a query

    Args:
        query:
        is_not:

    Returns:

    """
    if is_not:
        return invert_query(query.copy())
    else:
        return query


def build_int_criteria(path, comparison, value):
    """Builds a criteria for the type integer

    Args:
        path:
        comparison:
        value:

    Returns:

    """
    criteria = dict()

    if comparison == "=":
        criteria[path] = int(value)
    else:
        criteria[path] = json.loads('{{"${0}": {1} }}'.format(comparison, int(value)))

    return criteria


def build_float_criteria(path, comparison, value):
    """Builds a criteria for the type float

    Args:
        path:
        comparison:
        value:

    Returns:

    """
    criteria = dict()

    if comparison == "=":
        criteria[path] = float(value)
    else:
        criteria[path] = json.loads('{{"${0}": {1} }}'.format(comparison, float(value)))

    return criteria


def build_string_criteria(path, comparison, value):
    """Builds a criteria for the type string

    Args:
        path:
        comparison:
        value:

    Returns:

    """
    criteria = dict()

    if comparison == "is":
        criteria[path] = value
    elif comparison == "like":
        criteria[path] = "/" + value + "/"

    return criteria


def build_and_criteria(*criteria_list):
    """Builds a criteria that is the result of and operator for N criteria

    Args:
        criteria_list:

    Returns:

    """
    and_criteria = dict()
    and_criteria["$and"] = []

    for criteria in criteria_list:
        and_criteria["$and"].append(criteria)

    return and_criteria


def build_or_criteria(*criteria_list):
    """Builds a criteria that is the result of or operator for N criteria

    Args:
        criteria_list:

    Returns:

    """
    or_criteria = dict()
    or_criteria["$or"] = []

    for criteria in criteria_list:
        or_criteria["$or"].append(criteria)

    return or_criteria


def build_wildcard_elem_match_criteria(*criteria_list):
    # FIXME: Wildcard method should be move to the example type app
    # FIXME: Mongo_query could be an util class and be inherited as needed
    """Builds a criteria that is the result of N criteria elemMatch

    Args:
        criteria_list:

    Returns:

    """
    elem_match_criteria = dict()
    elem_match_criteria["$elemMatch"] = {}

    for criteria in criteria_list:
        elem_match_criteria["$elemMatch"].update(criteria)

    return {"list_content": elem_match_criteria}


def build_wildcard_criteria(criteria):
    # FIXME: Wildcard method should be move to the example type app
    # FIXME: Mongo_query could be an util class and be inherited as needed
    """Build wildcard criteria, from criteria.

    Args:
        criteria:

    Returns:

    """
    key, value = criteria.popitem()
    criteria_key = {"path": "/.*{}/".format(key)}
    criteria_value = {"value": value}
    return build_wildcard_elem_match_criteria(criteria_key, criteria_value)


def build_criteria(
    element_path,
    comparison,
    value,
    element_type,
    default_prefix,
    is_not=False,
    use_wildcard=False,
):
    """Looks at element type and route to the right function to build the criteria

    Args:
        element_path:
        comparison:
        value:
        element_type:
        default_prefix:
        is_not:
        use_wildcard:

    Returns:

    """
    element_query: List[Any] = []
    attribute_query: List[Any] = []
    # build the query: value can be found at element:value or at element.#text:value
    # second case appends when the element has attributes or namespace information
    if element_type in get_xsd_floating_numbers(default_prefix):
        element_query.append(build_float_criteria(element_path, comparison, value))
        attribute_query.append(
            build_float_criteria("{}.#text".format(element_path), comparison, value)
        )
    elif element_type in get_xsd_numbers(default_prefix):
        element_query.append(build_int_criteria(element_path, comparison, value))
        attribute_query.append(
            build_int_criteria("{}.#text".format(element_path), comparison, value)
        )
    elif element_type in get_xsd_gregorian_types(default_prefix):
        # before creating the int query check if the value type is int
        try:
            int_value = int(value)
            # if the format is number perform a strict match
            attribute_query.append(
                build_int_criteria("{}.#text".format(element_path), "=", int_value)
            )
            element_query.append(build_int_criteria(element_path, "=", int_value))
        except Exception:
            pass

        # string query
        element_query.append(build_string_criteria(element_path, comparison, value))
        attribute_query.append(
            build_string_criteria("{}.#text".format(element_path), comparison, value)
        )

    else:
        element_query.append(build_string_criteria(element_path, comparison, value))
        attribute_query.append(
            build_string_criteria("{}.#text".format(element_path), comparison, value)
        )

    if use_wildcard:
        element_query.append(build_wildcard_criteria(element_query))
        attribute_query.append(build_wildcard_criteria(attribute_query))

    # add a $or operator
    criteria = build_or_criteria(*(element_query + attribute_query))

    if is_not:
        return invert_query(criteria)
    else:
        return criteria


def invert_query(query):
    """Inverts each field of the query to build NOT(query)

    Args:
        query:

    Returns:

    """
    for key, value in list(query.items()):
        if key == "$and" or key == "$or":
            # Invert the query for the case value can be found at element:value or at
            # element.#text:value. Second case happens when the element has attributes
            # or namespace information.
            if (
                len(value) == 2
                and len(list(value[0].keys())) == 1
                and len(list(value[1].keys())) == 1
                and list(value[1].keys())[0]
                == "{}.#text".format(list(value[0].keys())[0])
            ):
                # second query is the same as the first
                if key == "$and":
                    return {"$or": [invert_query(value[0]), invert_query(value[1])]}
                elif key == "$or":
                    return {"$and": [invert_query(value[0]), invert_query(value[1])]}
            for sub_value in value:
                invert_query(sub_value)
        else:
            # lt, lte, =, gte, gt, not, ne
            if isinstance(value, dict):
                if list(value.keys())[0] == "$not" or list(value.keys())[0] == "$ne":
                    query[key] = value[list(value.keys())[0]]
                else:
                    saved_value = value
                    query[key] = dict()
                    query[key]["$not"] = saved_value
            else:
                saved_value = value
                if is_regex(value):
                    query[key] = dict()
                    query[key]["$not"] = saved_value
                else:
                    query[key] = dict()
                    query[key]["$ne"] = saved_value
    return query


def is_regex(expr):
    """Returns true if the expression is a regular expression

    Args:
        expr:

    Returns:

    """
    if isinstance(expr, re.Pattern):
        return True

    try:
        return expr.startswith("/") and expr.endswith("/")
    except:
        return False


def get_dot_notation_to_element(data_structure_element, namespaces):
    """

    Args:
        data_structure_element:
        namespaces:

    Returns:

    """
    # get data structure element's xml xpath
    xml_xpath = data_structure_element.options["xpath"]["xml"]
    # transform xml xpath to dot notation
    dot_notation = xpath_to_dot_notation(xml_xpath, namespaces)

    return dot_notation


def get_parent_name(data_structure_element_id, namespaces, request):
    """

    Args:
        data_structure_element_id:
        namespaces:
        request:

    Returns:

    """
    # get the data structure element
    data_structure_element = data_structure_element_api.get_by_id(
        data_structure_element_id, request
    )
    # convert xml path to mongo dot notation
    data_structure_element_dot_notation = get_dot_notation_to_element(
        data_structure_element, namespaces
    )
    # get parent name
    parent_name = data_structure_element_dot_notation.split(".")[-2]

    return parent_name


def get_parent_path(data_structure_element_id, namespaces, request):
    """

    Args:
        data_structure_element_id:
        namespaces:
        request:

    Returns:

    """
    # get the data structure element
    data_structure_element = data_structure_element_api.get_by_id(
        data_structure_element_id, request
    )
    # convert xml path to mongo dot notation
    data_structure_element_dot_notation = get_dot_notation_to_element(
        data_structure_element, namespaces
    )
    # get path to parent element
    parent_path = ".".join(data_structure_element_dot_notation.split(".")[:-1])

    return parent_path


def check_query_form(form_values, template_id, request=None):
    """Checks that values entered by the user match each element type

    Args:
        form_values:
        template_id:
        request:

    Returns:

    """
    template = template_api.get_by_id(template_id, request=request)
    namespaces = get_namespaces(template.content)
    default_prefix = get_default_prefix(namespaces)

    # check if there are no errors in the query
    errors = []

    if len(form_values) == 0:
        errors.append("The query is empty.")

    for field in form_values:
        element_value = get_element_value(field)
        element_name = field.get("name", "Unnamed field")
        element_type = field.get("type", None)
        # If there is a type to check
        if element_type:
            error = validate_element_value(
                element_name, element_type, element_value, default_prefix
            )
            if error is not None:
                errors.append(error)

    return errors


def fields_to_query(form_values, template_id, use_wildcard=False, request=None):
    """Takes values from the html tree and creates a query from them

    Args:
        form_values:
        template_id:
        use_wildcard:
        request:
    Returns:

    """
    # get template
    template = template_api.get_by_id(template_id, request=request)
    # get namespaces
    namespaces = get_namespaces(template.content)
    # get default prefix
    default_prefix = get_default_prefix(namespaces)

    query = dict()
    for field in form_values:
        bool_comp = field["operator"]
        is_not = bool_comp == "NOT"
        element_type = field.get("type", None)

        # get element value
        value = get_element_value(field)
        # get comparison operator
        comparison = get_element_comparison(field)

        element_id = field["id"]

        if element_type == "query":
            try:
                saved_query = saved_query_api.get_by_id(element_id)
            except DoesNotExist:
                raise MongoQueryException("The saved query does not exist anymore.")
            criteria = build_query_criteria(json.loads(saved_query.query), is_not)
        else:
            data_structure_element = data_structure_element_api.get_by_id(
                element_id, request
            )
            element = get_dot_notation_to_element(data_structure_element, namespaces)
            criteria = build_criteria(
                element,
                comparison,
                value,
                element_type,
                default_prefix,
                is_not,
                use_wildcard,
            )

        if bool_comp == "OR":
            query = build_or_criteria(query, criteria)
        elif bool_comp == "AND":
            query = build_and_criteria(query, criteria)
        else:
            if form_values.index(field) == 0:
                query.update(criteria)
            else:
                query = build_and_criteria(query, criteria)

    return query


def sub_elements_to_query(form_values, namespaces, default_prefix, request):
    """Transforms HTML fields in a query on sub-elements

    Args:
        form_values:
        namespaces:
        default_prefix:
        request:

    Returns:

    """
    elem_match = []

    # get the parent path using the first element of the list
    parent_path = get_parent_path(form_values[0]["id"], namespaces, request)

    for i in range(0, len(form_values)):
        field = form_values[i]
        if field["selected"] is True:
            bool_comp = field["operator"]
            if bool_comp == "NOT":
                is_not = True
            else:
                is_not = False

            data_structure_element = data_structure_element_api.get_by_id(
                field["id"], request
            )
            element_type = data_structure_element.options["type"]
            element_name = data_structure_element.options["name"]
            value = get_element_value(field)
            comparison = get_element_comparison(field)

            criteria = build_criteria(
                element_name, comparison, value, element_type, default_prefix, is_not
            )

            elem_match.append(criteria)

    query = {parent_path: {"$elemMatch": {"$and": elem_match}}}

    return query



""" Url router for the explore example application
"""
from django.conf.urls import include
from django.urls import re_path

from core_explore_example_app.utils.mongo_query import fields_to_query
from core_explore_example_app.views.user import views as user_views, ajax as user_ajax

urlpatterns = [
    re_path(r"^$", user_views.IndexView.as_view(), name="core_explore_example_index"),
    re_path(r"^rest/", include("core_explore_example_app.rest.urls")),
    re_path(
        r"^select-fields/(?P<template_id>\w+)$",
        user_views.SelectFieldsView.as_view(),
        name="core_explore_example_select_fields",
    ),
    re_path(
        r"^build-query/(?P<template_id>\w+)$",
        user_views.BuildQueryView.as_view(),
        name="core_explore_example_build_query",
    ),
    re_path(
        r"^build-query/(?P<template_id>\w+)/(?P<query_id>\w+)$",
        user_views.BuildQueryView.as_view(),
        name="core_explore_example_build_query",
    ),
    re_path(
        r"^results/(?P<template_id>\w+)/(?P<query_id>\w+)$",
        user_views.ResultQueryView.as_view(),
        name="core_explore_example_results",
    ),
    re_path(
        r"^save-fields$", user_ajax.save_fields, name="core_explore_example_save_fields"
    ),
    re_path(
        r"^generate-element/(?P<explore_data_structure_id>\w+)$",
        user_ajax.generate_element,
        name="core_explore_example_generate_element",
    ),
    re_path(
        r"^generate-choice/(?P<explore_data_structure_id>\w+)$",
        user_ajax.generate_choice,
        name="core_explore_example_generate_choice",
    ),
    re_path(
        r"^remove-element$",
        user_ajax.remove_element,
        name="core_explore_example_remove_element",
    ),
    re_path(
        r"^select-element$",
        user_ajax.select_element,
        name="core_explore_example_select_element",
    ),
    re_path(
        r"^update-user-input$",
        user_ajax.update_user_input,
        name="core_explore_example_update_user_input",
    ),
    re_path(
        r"^get-sub-elements-query-builder$",
        user_ajax.get_sub_elements_query_builder,
        name="core_explore_example_get_sub_elements_query_builder",
    ),
    re_path(
        r"^insert-sub-elements-query$",
        user_ajax.insert_sub_elements_query,
        name="core_explore_example_insert_sub_elements_query",
    ),
    re_path(
        r"^add-criteria$",
        user_ajax.add_criteria,
        name="core_explore_example_add_criteria",
    ),
    re_path(
        r"^save-query$",
        user_ajax.SaveQueryView.as_view(fields_to_query_func=fields_to_query),
        name="core_explore_example_save_query",
    ),
    re_path(
        r"^clear-criteria$",
        user_ajax.clear_criteria,
        name="core_explore_example_clear_criteria",
    ),
    re_path(
        r"^clear-queries$",
        user_ajax.clear_queries,
        name="core_explore_example_clear_queries",
    ),
    re_path(
        r"^delete-query$",
        user_ajax.delete_query,
        name="core_explore_example_delete_query",
    ),
    re_path(
        r"^add-query-criteria$",
        user_ajax.add_query_criteria,
        name="core_explore_example_add_query_criteria",
    ),
    re_path(
        r"^get-query$",
        user_ajax.GetQueryView.as_view(fields_to_query_func=fields_to_query),
        name="core_explore_example_get_query",
    ),
    re_path(
        r"^get-persistent-query-url$",
        user_ajax.CreatePersistentQueryExampleUrlView.as_view(),
        name="core_explore_example_get_persistent_query_url",
    ),
    re_path(
        r"^results-redirect",
        user_views.ResultQueryExampleRedirectView.as_view(),
        name="core_explore_example_results_redirect",
    ),
]



""" Migrations
"""
# Generated by Django 3.2 on 2021-12-08 21:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    """Migration"""

    initial = True

    dependencies = [
        ("core_main_app", "0001_initial"),
        ("core_parser_app", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="ExploreDataStructure",
            fields=[
                (
                    "datastructure_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core_parser_app.datastructure",
                    ),
                ),
                (
                    "selected_fields_html_tree",
                    models.TextField(blank=True, default=None, null=True),
                ),
            ],
            bases=("core_parser_app.datastructure",),
        ),
        migrations.CreateModel(
            name="ExploreExample",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
            ],
            options={
                "verbose_name": "core_explore_example_app",
                "permissions": (
                    ("access_explore_example", "Can access explore example"),
                    ("save_query", "Can save query"),
                    ("delete_query", "Can delete query"),
                    (
                        "access_explore_example_data_structure",
                        "Can access explore example data structure",
                    ),
                ),
                "default_permissions": (),
            },
        ),
        migrations.CreateModel(
            name="SavedQuery",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("user_id", models.CharField(max_length=200)),
                ("query", models.TextField()),
                ("displayed_query", models.TextField()),
                (
                    "template",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="core_main_app.template",
                    ),
                ),
            ],
            options={
                "verbose_name": "Saved Query",
                "verbose_name_plural": "Saved Queries",
            },
        ),
        migrations.CreateModel(
            name="PersistentQueryExample",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("user_id", models.CharField(max_length=200)),
                ("content", models.TextField(blank=True, null=True)),
                ("data_sources", models.JSONField(blank=True, default=list)),
                ("creation_date", models.DateTimeField(auto_now_add=True)),
                (
                    "name",
                    models.CharField(
                        blank=True, max_length=200, null=True, unique=True
                    ),
                ),
                (
                    "templates",
                    models.ManyToManyField(
                        blank=True, default=[], to="core_main_app.Template"
                    ),
                ),
            ],
            options={
                "abstract": False,
                "verbose_name": "Persistent Query by Example",
                "verbose_name_plural": "Persistent Queries by Example",
            },
        ),
    ]



""" Unit tests for PersistentQueryExample.
"""
from unittest import TestCase, mock

from mock import patch

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_explore_example_app.components.persistent_query_example import (
    api as persistent_query_example_api,
)
from core_explore_example_app.components.persistent_query_example.models import (
    PersistentQueryExample,
)


class TestPersistentQueryExampleGetById(TestCase):
    """Test Persistent Query Example Get By Id"""

    @patch.object(PersistentQueryExample, "get_by_id")
    def test_persistent_query_example_get_by_id_return_data_if_found(
        self, mock_get_by_id
    ):
        """test_persistent_query_example_get_by_id_return_data_if_found"""

        # Arrange
        expected_result = PersistentQueryExample(user_id="1")
        mock_get_by_id.return_value = expected_result
        mock_user = create_mock_user("1")

        # Act # Assert
        self.assertEqual(
            persistent_query_example_api.get_by_id("mock_id", mock_user),
            expected_result,
        )

    def test_persistent_query_example_get_by_id_raises_model_error_if_not_found(self):
        """test_persistent_query_example_get_by_id_raises_model_error_if_not_found"""

        # Arrange
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(exceptions.ModelError):
            persistent_query_example_api.get_by_id("mock_id", mock_user)

    @patch.object(PersistentQueryExample, "get_by_id")
    def test_persistent_query_example_get_by_id_raises_does_not_exist_error_if_not_found(
        self, mock_get_by_id
    ):
        """test_persistent_query_example_get_by_id_raises_does_not_exist_error_if_not_found"""

        # Arrange
        mock_get_by_id.side_effect = exceptions.DoesNotExist(message="mock error")
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            persistent_query_example_api.get_by_id("mock_id", mock_user)


class TestsPersistentQueryExampleGetByName(TestCase):
    """Tests Persistent Query Example Get By Name"""

    @mock.patch.object(PersistentQueryExample, "get_by_name")
    def test_persistent_query_example_get_by_name_return_data_if_found(
        self, mock_get_by_name
    ):
        """test_persistent_query_example_get_by_name_return_data_if_found"""
        # Arrange
        expected_result = PersistentQueryExample(user_id="1")
        mock_get_by_name.return_value = expected_result
        mock_user = create_mock_user("1")

        # Act # Assert
        self.assertEqual(
            persistent_query_example_api.get_by_name("mock_name", mock_user),
            expected_result,
        )

    @patch.object(PersistentQueryExample, "get_by_name")
    def test_persistent_query_example_get_by_name_raises_does_not_exist_error_if_not_found(
        self, mock_get_by_name
    ):
        """test_persistent_query_example_get_by_name_raises_does_not_exist_error_if_not_found"""

        # Arrange
        mock_get_by_name.side_effect = exceptions.DoesNotExist(message="mock error")
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            persistent_query_example_api.get_by_name("mock_id", mock_user)


class TestsPersistentQueryExampleUpsert(TestCase):
    """Tests Persistent Query Example Upsert"""

    def setUp(self) -> None:
        self.mock_persistent_query_example = PersistentQueryExample(
            user_id="1",
            name="mock_example",
            content={"content_test"},
        )

    @patch.object(PersistentQueryExample, "save")
    def test_persistent_query_example_upsert_return_data(self, mock_save):
        """test_persistent_query_example_upsert_return_data"""

        # Arrange
        mock_save.return_value = self.mock_persistent_query_example
        mock_user = create_mock_user("1")

        # Act
        result = persistent_query_example_api.upsert(
            self.mock_persistent_query_example, mock_user
        )

        # Assert
        self.assertIsInstance(result, PersistentQueryExample)


class TestsPersistentQueryExampleDelete(TestCase):
    """Tests Persistent Query Example Delete"""

    @patch.object(PersistentQueryExample, "delete")
    def test_returns_no_error(self, mock_delete):
        """test_returns_no_error"""

        # Arrange
        mock_delete.return_value = None
        mock_user = create_mock_user("1")

        # Act # Assert
        self.assertEqual(
            persistent_query_example_api.delete(
                PersistentQueryExample(user_id="1"), mock_user
            ),
            None,
        )


class TestsPersistentQueryExampleGetAll(TestCase):
    """Tests Persistent Query Example Get All"""

    @patch.object(PersistentQueryExample, "get_all")
    def test_returns_no_error(self, mock_get_all):
        """test_returns_no_error"""

        # Arrange
        expected_result = {
            PersistentQueryExample(id=1, user_id="1"),
            PersistentQueryExample(id=2, user_id="2"),
        }
        mock_get_all.return_value = expected_result

        mock_user = create_mock_user("1", is_superuser=True, is_staff=True)

        # Act # Assert
        self.assertEqual(
            persistent_query_example_api.get_all(mock_user), expected_result
        )

    @patch.object(PersistentQueryExample, "get_all")
    def test_persistent_query_example_get_all_raises_does_not_access_control_error_if_not_admin(
        self, mock_get_all
    ):
        """test_persistent_query_example_get_all_raises_does_not_access_control_error_if_not_admin"""

        # Arrange
        mock_get_all.side_effect = AccessControlError
        mock_user = create_mock_user("1")

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_example_api.get_all(mock_user)


class TestsPersistentQueryExampleGetAllByUser(TestCase):
    """Tests Persistent Query Example Get All By User"""

    @patch.object(PersistentQueryExample, "get_all_by_user")
    def test_returns_no_error(self, mock_get_all_by_user):
        """test_returns_no_error"""

        # Arrange
        expected_result = {
            PersistentQueryExample(id=1, user_id="1"),
            PersistentQueryExample(id=2, user_id="1"),
        }
        mock_get_all_by_user.return_value = expected_result

        mock_user = create_mock_user("1")

        # Act # Assert
        self.assertEqual(
            persistent_query_example_api.get_all_by_user(mock_user), expected_result
        )



""" System API
"""
from core_explore_example_app.apps import ExploreExampleAppConfig
from core_explore_example_app.components.saved_query.models import SavedQuery


def get_saved_queries_created_by_app():
    """Return saved queries created by the app.

    Returns:

    """
    return SavedQuery.objects.filter(user_id=ExploreExampleAppConfig.name).all()



""" REST views for the Persistent Query Example.
"""
from django.db import IntegrityError
from rest_framework import status
from rest_framework.exceptions import ValidationError
from rest_framework.permissions import IsAdminUser, IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView

from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.commons import exceptions
import core_explore_example_app.components.persistent_query_example.api as persistent_query_example_api
from core_explore_example_app.rest.persistent_query_example.serializers import (
    PersistentQueryExampleSerializer,
    PersistentQueryExampleAdminSerializer,
)


class AdminPersistentQueryExampleList(APIView):
    """List all persistent query example, or create a new one."""

    permission_classes = (IsAdminUser,)
    serializer = PersistentQueryExampleAdminSerializer

    def get(self, request):
        """Get all user persistent query example

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of persistent query example
            - code: 403
              content: Forbidden
            - code: 500
              content: Internal server error
        """
        if not request.user.is_superuser:
            return Response(status=status.HTTP_403_FORBIDDEN)

        try:
            # Get object
            object_list = persistent_query_example_api.get_all(request.user)

            # Serialize object
            serializer = self.serializer(object_list, many=True)

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def post(self, request):
        """Create a persistent query example

        Parameters:

            {
                "content": "{}",
                "templates": ["5ea99316d26ebc48e475c60a"],
                "name": "persistent_query_example"
            }

        Args:

            request: HTTP request

        Returns:

            - code: 201
              content: Created persistent query example
            - code: 400
              content: Validation error
            - code: 403
              content: Forbidden
            - code: 500
              content: Internal server error
        """
        if not request.user.is_superuser:
            return Response(status=status.HTTP_403_FORBIDDEN)

        try:
            # Build serializer
            serializer = self.serializer(
                data=request.data, context={"request": request}
            )

            # Validate data
            serializer.is_valid(True)
            # Save data
            serializer.save()

            # Return the serialized data
            return Response(serializer.data, status=status.HTTP_201_CREATED)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class PersistentQueryExampleList(APIView):
    """List all persistent query example or create one"""

    permission_classes = (IsAuthenticated,)
    serializer = PersistentQueryExampleSerializer

    def get(self, request):
        """Get all user persistent query example

        Args:

            request: HTTP request

        Returns:

            - code: 200
              content: List of persistent query example
            - code: 403
              content: Forbidden
            - code: 500
              content: Internal server error
        """

        try:
            # Get object
            object_list = persistent_query_example_api.get_all_by_user(request.user)

            # Serialize object
            serializer = self.serializer(object_list, many=True)

            # Return response
            return Response(serializer.data, status=status.HTTP_200_OK)
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def post(self, request):
        """Create a new persistent query example

        Parameters:

            {
                "content": "{}",
                "templates": ["5ea99316d26ebc48e475c60a"],
                "name": "persistent_query_example"
            }

        Args:

            request: HTTP request

        Returns:

            - code: 201
              content: Created data
            - code: 400
              content: Validation error
            - code: 500
              content: Internal server error
        """

        try:
            # Build serializer
            serializer = self.serializer(
                data=request.data, context={"request": request}
            )

            # Validate data
            serializer.is_valid(True)
            # Save data
            serializer.save()

            # Return the serialized data
            return Response(serializer.data, status=status.HTTP_201_CREATED)

        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class PersistentQueryExampleDetail(APIView):
    """Persistent query example detail"""

    permission_classes = (IsAuthenticated,)
    serializer = PersistentQueryExampleSerializer

    def get(self, request, pk):
        """Retrieve persistent query example from database

        Args:

            request: HTTP request
            pk: ObjectId

        Returns:
            - code: 200
              content: persistent query example
            - code: 403
              content: Forbidden
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            persistent_query_example = persistent_query_example_api.get_by_id(
                pk, request.user
            )

            # Serialize object
            serializer = self.serializer(persistent_query_example)

            # Return response
            return Response(serializer.data)

        except exceptions.DoesNotExist:
            content = {"message": "Object not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def patch(self, request, pk):
        """Update a persistent query example

        Parameters:
            {
                "content": "{}",
                "templates": ["5ea99316d26ebc48e475c60a"],
                "name": null
            }

        Args:
            request: HTTP request
            pk: ObjectId

        Returns:
            - code: 200
              content: Updated data
            - code: 400
              content: Validation error
            - code: 403
              content: Forbidden
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            persistent_query_example = persistent_query_example_api.get_by_id(
                pk, request.user
            )

            # Build serializer
            serializer = self.serializer(
                instance=persistent_query_example,
                data=request.data,
                partial=True,
                context={"request": request},
            )

            # Validate and save persistent query example
            serializer.is_valid(True)
            serializer.save()

            return Response(serializer.data, status=status.HTTP_200_OK)
        except ValidationError as validation_exception:
            content = {"message": validation_exception.detail}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except IntegrityError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_400_BAD_REQUEST)
        except exceptions.DoesNotExist:
            content = {"message": "Persistent query example not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

    def delete(self, request, pk):
        """Delete a persistent query example

        Args:
            request: HTTP request
            pk: ObjectId

        Returns:
            - code: 204
              content: Deletion success
            - code: 403
              content: Authentication error
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            persistent_query_example = persistent_query_example_api.get_by_id(
                pk, request.user
            )

            # delete object
            persistent_query_example_api.delete(persistent_query_example, request.user)

            # Return response
            return Response(status=status.HTTP_204_NO_CONTENT)
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except exceptions.DoesNotExist:
            content = {"message": "Persistent query example not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)


class PersistentQueryExampleByName(APIView):
    """Persistent query example detail"""

    permission_classes = (IsAuthenticated,)
    serializer = PersistentQueryExampleSerializer

    def get(self, request, name):
        """Retrieve persistent query example from database

        Args:

            request: HTTP request
            name: name

        Returns:
            - code: 200
              content: persistent query example
            - code: 403
              content: Forbidden
            - code: 404
              content: Object was not found
            - code: 500
              content: Internal server error
        """
        try:
            # Get object
            persistent_query_example = persistent_query_example_api.get_by_name(
                name, request.user
            )

            # Serialize object
            serializer = self.serializer(persistent_query_example)

            # Return response
            return Response(serializer.data)
        except exceptions.DoesNotExist:
            content = {"message": "Persistent query example not found."}
            return Response(content, status=status.HTTP_404_NOT_FOUND)
        except AccessControlError as exception:
            content = {"message": str(exception)}
            return Response(content, status=status.HTTP_403_FORBIDDEN)
        except Exception as api_exception:
            content = {"message": str(api_exception)}
            return Response(content, status=status.HTTP_500_INTERNAL_SERVER_ERROR)



""" Serializers used for the persistent query example REST API.
"""
from rest_framework.serializers import ModelSerializer

from core_explore_example_app.components.persistent_query_example import (
    api as persistent_query_example_api,
)
from core_explore_example_app.components.persistent_query_example.models import (
    PersistentQueryExample,
)


class PersistentQueryExampleSerializer(ModelSerializer):
    """Persistent query example"""

    class Meta:
        """Meta"""

        model = PersistentQueryExample
        fields = ["id", "user_id", "content", "templates", "name"]
        read_only_fields = ("id", "user_id")

    def create(self, validated_data):
        """Create and return a new `PersistentQueryExample` instance, given the validated data."""

        # Create instance from the validated data and insert it in DB
        persistent_query_example = PersistentQueryExample(
            user_id=str(self.context["request"].user.id),
            content=validated_data["content"] if "content" in validated_data else None,
            name=validated_data["name"] if "name" in validated_data else None,
        )
        persistent_query_example_api.upsert(
            persistent_query_example, self.context["request"].user
        )

        if "templates" in validated_data:
            persistent_query_example.templates.set(validated_data["templates"])
        return persistent_query_example

    # Update instance from the validated data and insert it in DB
    def update(self, persistent_query_example, validated_data):
        """Update and return an existing `PersistentQueryExample` instance, given the validated
        data.
        """
        persistent_query_example.content = validated_data.get(
            "content", persistent_query_example.content
        )
        persistent_query_example.name = validated_data.get(
            "name", persistent_query_example.name
        )
        persistent_query_example_api.upsert(
            persistent_query_example, self.context["request"].user
        )
        if "templates" in validated_data:
            persistent_query_example.templates.set(validated_data["templates"])
        return persistent_query_example


class PersistentQueryExampleAdminSerializer(ModelSerializer):
    """PersistentQueryAdminExample Serializer"""

    class Meta:
        """Meta"""

        model = PersistentQueryExample
        fields = ["id", "user_id", "content", "templates", "name"]

    def create(self, validated_data):
        """
        Create and return a new `PersistentQueryExample` instance, given the validated data.
        """
        # Create data
        persistent_query_example = PersistentQueryExample(
            user_id=validated_data["user_id"],
            content=validated_data["content"] if "content" in validated_data else None,
            name=validated_data["name"] if "name" in validated_data else None,
        )
        persistent_query_example_api.upsert(
            persistent_query_example, self.context["request"].user
        )
        if "templates" in validated_data:
            persistent_query_example.templates.set(validated_data["templates"])
        return persistent_query_example



""" Unit Test Saved Query
"""
from unittest.case import TestCase

from mock import patch
from mock.mock import Mock

from core_main_app.commons import exceptions
from core_main_app.components.template.models import Template
from core_explore_example_app.components.saved_query import api as saved_query_api
from core_explore_example_app.components.saved_query.models import SavedQuery


class TestSavedQueryGetById(TestCase):
    """Test Saved Query Get By Id"""

    @patch.object(SavedQuery, "get_by_id")
    def test_saved_query_get_by_id_raises_api_error_if_not_found(self, mock_get):
        """test_saved_query_get_by_id_raises_api_error_if_not_found"""

        # Arrange
        mock_get.side_effect = exceptions.DoesNotExist("")
        # Act # Assert
        with self.assertRaises(exceptions.DoesNotExist):
            saved_query_api.get_by_id(1)

    @patch.object(SavedQuery, "get_by_id")
    def test_saved_query_get_by_id_returns_saved_query_if_found(self, mock_get):
        """test_saved_query_get_by_id_returns_saved_query_if_found"""

        # Arrange
        mock_data = _create_saved_query()
        mock_get.return_value = mock_data
        # Act
        result = saved_query_api.get_by_id(mock_data.id)
        # Assert
        self.assertIsInstance(result, SavedQuery)


class TestSavedQueryGetAllByUserAndTemplate(TestCase):
    """Test Saved Query Get All By User And Template"""

    @patch.object(SavedQuery, "get_all_by_user_and_template")
    def test_saved_queries_get_all_by_user_and_template_returns_saved_queries_from_user(
        self, mock_get_by_user_and_template
    ):
        """test_saved_queries_get_all_by_user_and_template_returns_saved_queries_from_user"""

        user_id = 1
        template_id = 1
        template = Template(id=1)
        mock_data_1 = _create_saved_query(user_id=user_id, template=template)
        mock_data_2 = _create_saved_query(user_id=user_id, template=template)
        mock_get_by_user_and_template.return_value = [mock_data_1, mock_data_2]
        # Act
        result = saved_query_api.get_all_by_user_and_template(
            user_id, template_id=template_id
        )
        # Assert
        self.assertTrue(all(item.user_id == user_id for item in result))


def _create_saved_query(user_id=None, template=None):
    """Returns a saved query

    Args:
        user_id:
        template:

    Returns:

    """
    if user_id is None:
        user_id = 1
    if template is None:
        template = Template()

    return SavedQuery(
        id=1,
        user_id=user_id,
        template=template,
        query="{}",
        displayed_query="query",
    )


def _create_mock_saved_query(user_id=None, template=None):
    """Returns a mock saved query

    Args:
        user_id:
        template:

    Returns:

    """
    if user_id is None:
        user_id = 1
    if template is None:
        template = Template()
    mock_saved_query = Mock(spec=SavedQuery)
    mock_saved_query.user_id = user_id
    mock_saved_query.template = template
    mock_saved_query.id = 1

    return mock_saved_query



""" Url router for the REST API
"""
from django.urls import re_path
from rest_framework.urlpatterns import format_suffix_patterns

from core_explore_example_app.rest.persistent_query_example import (
    views as persistent_query_example_views,
)
from core_explore_example_app.rest.saved_query import views as saved_query_views

urlpatterns = [
    re_path(
        r"^saved/query/$",
        saved_query_views.SavedQueryList.as_view(),
        name="core_explore_example_app_rest_saved_query_list",
    ),
    re_path(
        r"^saved/query/(?P<pk>\w+)/$",
        saved_query_views.SavedQueryDetail.as_view(),
        name="core_explore_example_app_rest_saved_query_detail",
    ),
    re_path(
        r"^admin/persistent_query_example/$",
        persistent_query_example_views.AdminPersistentQueryExampleList.as_view(),
        name="core_explore_example_app_rest_persistent_query_example_admin_list",
    ),
    re_path(
        r"^persistent_query_example/$",
        persistent_query_example_views.PersistentQueryExampleList.as_view(),
        name="core_explore_example_app_rest_persistent_query_example_list",
    ),
    re_path(
        r"^persistent_query_example/(?P<pk>\w+)/$",
        persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
        name="core_explore_example_app_rest_persistent_query_example_detail",
    ),
    re_path(
        r"^persistent_query_example/name/(?P<name>\w+)/$",
        persistent_query_example_views.PersistentQueryExampleByName.as_view(),
        name="core_explore_example_app_rest_persistent_query_example_name",
    ),
]

urlpatterns = format_suffix_patterns(urlpatterns)



""" Custom admin site for the Persistent Query Example model
"""
from django.contrib import admin


class CustomPersistentQueryExampleAdmin(admin.ModelAdmin):
    """CustomPersistentQueryExampleAdmin"""

    exclude = ["data_sources"]

    def has_add_permission(self, request, obj=None):
        """Prevent from manually adding Persistent Queries"""
        return False



"""Saved query model
"""
from django.core.exceptions import ObjectDoesNotExist
from django.db import models

from core_main_app.commons import exceptions
from core_main_app.components.template.models import Template


class SavedQuery(models.Model):
    """Represents a query saved by the user (Query by Example)"""

    user_id = models.CharField(blank=False, max_length=200)
    template = models.ForeignKey(Template, on_delete=models.CASCADE)
    query = models.TextField(blank=False)
    displayed_query = models.TextField(blank=False)

    class Meta:
        """Meta"""

        verbose_name = "Saved Query"
        verbose_name_plural = "Saved Queries"

    @staticmethod
    def get_all():
        """Get all Saved Query.

        Args:

        Returns:

        """
        return SavedQuery.objects.all()

    @staticmethod
    def get_by_id(query_id):
        """Get a saved query

        Args:
            query_id:

        Returns:

        """
        try:
            return SavedQuery.objects.get(pk=query_id)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_all_by_user_and_template(user_id, template_id):
        """Gets a saved query by user id and template id

        Args:
            user_id:
            template_id:

        Returns:

        """
        return SavedQuery.objects.filter(user_id=str(user_id), template=template_id)



""" Authentication tests for PersistentQueryExample REST API.
"""
from unittest.mock import patch

from django.contrib.auth.models import AnonymousUser
from django.test import SimpleTestCase
from rest_framework import status

from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
import core_explore_example_app.components.persistent_query_example.api as persistent_query_example_api
from core_explore_example_app.components.persistent_query_example.models import (
    PersistentQueryExample,
)
from core_explore_example_app.rest.persistent_query_example import (
    views as persistent_query_example_views,
)
from core_explore_example_app.rest.persistent_query_example.serializers import (
    PersistentQueryExampleSerializer,
    PersistentQueryExampleAdminSerializer,
)


class TestAdminPersistentQueryExampleListGet(SimpleTestCase):
    """Test Admin Persistent Query Example List Get"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_get(
            persistent_query_example_views.AdminPersistentQueryExampleList.as_view(),
            AnonymousUser(),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403"""

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            persistent_query_example_views.AdminPersistentQueryExampleList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(PersistentQueryExample, "get_all")
    def test_superuser_returns_http_200(self, get_all):
        """test_superuser_returns_http_200"""

        get_all.return_value = {}
        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_get(
            persistent_query_example_views.AdminPersistentQueryExampleList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPersistentQueryExampleListGet(SimpleTestCase):
    """Test Persistent Query Example List Get"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleList.as_view(),
            AnonymousUser(),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(PersistentQueryExample, "get_all_by_user")
    def test_authenticated_returns_http_200(self, get_all):
        """test_authenticated_returns_http_200"""

        get_all.return_value = {}
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(PersistentQueryExample, "get_all_by_user")
    def test_superuser_returns_http_200(self, get_all):
        """test_superuser_returns_http_200"""

        get_all.return_value = {}
        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPersistentQueryExampleListPost(SimpleTestCase):
    """Test Persistent Query Example List Post"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_post(
            persistent_query_example_views.AdminPersistentQueryExampleList.as_view(),
            AnonymousUser(),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(PersistentQueryExampleSerializer, "is_valid")
    @patch.object(PersistentQueryExampleSerializer, "save")
    @patch.object(PersistentQueryExampleSerializer, "data")
    def test_authenticated_returns_http_201(
        self,
        persistent_query_example_serializer_data,
        persistent_query_example_serializer_save,
        persistent_query_example_serializer_is_valid,
    ):
        """test_authenticated_returns_http_201"""

        persistent_query_example_serializer_is_valid.return_value = True
        persistent_query_example_serializer_save.return_value = None
        persistent_query_example_serializer_data.return_value = {}
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            persistent_query_example_views.PersistentQueryExampleList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    @patch.object(PersistentQueryExampleAdminSerializer, "is_valid")
    @patch.object(PersistentQueryExampleAdminSerializer, "save")
    @patch.object(PersistentQueryExampleAdminSerializer, "data")
    def test_superuser_returns_http_201(
        self,
        persistent_query_example_serializer_data,
        persistent_query_example_serializer_save,
        persistent_query_example_serializer_is_valid,
    ):
        """test_superuser_returns_http_201"""

        persistent_query_example_serializer_is_valid.return_value = True
        persistent_query_example_serializer_save.return_value = None
        persistent_query_example_serializer_data.return_value = {}

        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_post(
            persistent_query_example_views.AdminPersistentQueryExampleList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestAdminPersistentQueryExampleListPost(SimpleTestCase):
    """Test Admin Persistent Query Example List Post"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_post(
            persistent_query_example_views.AdminPersistentQueryExampleList.as_view(),
            AnonymousUser(),
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_authenticated_returns_http_403(self):
        """test_authenticated_returns_http_403"""

        mock_user = create_mock_user("1")

        response = RequestMock.do_request_post(
            persistent_query_example_views.AdminPersistentQueryExampleList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(PersistentQueryExampleAdminSerializer, "is_valid")
    @patch.object(PersistentQueryExampleAdminSerializer, "save")
    @patch.object(PersistentQueryExampleAdminSerializer, "data")
    def test_superuser_returns_http_201(
        self,
        persistent_query_example_serializer_data,
        persistent_query_example_serializer_save,
        persistent_query_example_serializer_is_valid,
    ):
        """test_superuser_returns_http_201"""

        persistent_query_example_serializer_is_valid.return_value = True
        persistent_query_example_serializer_save.return_value = None
        persistent_query_example_serializer_data.return_value = {}

        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        response = RequestMock.do_request_post(
            persistent_query_example_views.AdminPersistentQueryExampleList.as_view(),
            mock_user,
        )

        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestPersistentQueryExampleDetailGet(SimpleTestCase):
    """Test Persistent Query Example Detail Get"""

    @patch.object(persistent_query_example_api, "get_by_id")
    @patch.object(PersistentQueryExampleSerializer, "data")
    def test_anonymous_returns_http_403(
        self,
        mock_persistent_query_example_serializer_data,
        mock_persistent_query_example_api_get_by_id,
    ):
        """test_anonymous_returns_http_403"""
        mock_persistent_query_example_serializer_data.return_value = {}
        mock_persistent_query_example_api_get_by_id.return_value = None
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            AnonymousUser(),
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(persistent_query_example_api, "get_by_id")
    @patch.object(PersistentQueryExampleSerializer, "data")
    def test_authenticated_returns_http_200(
        self,
        mock_persistent_query_example_serializer_data,
        mock_persistent_query_example_api_get_by_id,
    ):
        """test_authenticated_returns_http_200"""
        mock_persistent_query_example_serializer_data.return_value = {}
        mock_persistent_query_example_api_get_by_id.return_value = None
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(persistent_query_example_api, "get_by_id")
    @patch.object(PersistentQueryExampleSerializer, "data")
    def test_superuser_returns_http_200(
        self,
        mock_persistent_query_example_serializer_data,
        mock_persistent_query_example_api_get_by_id,
    ):
        """test_superuser_returns_http_200"""
        mock_persistent_query_example_serializer_data.return_value = {}
        mock_persistent_query_example_api_get_by_id.return_value = None
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPersistentQueryExampleDetailPatch(SimpleTestCase):
    """Test Persistent Query Example Detail Patch"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_patch(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            AnonymousUser(),
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(persistent_query_example_api, "get_by_id")
    @patch.object(PersistentQueryExampleSerializer, "is_valid")
    @patch.object(PersistentQueryExampleSerializer, "save")
    @patch.object(PersistentQueryExampleSerializer, "data")
    def test_authenticated_returns_http_200(
        self,
        mock_persistent_query_example_data,
        mock_persistent_query_example_save,
        mock_persistent_query_example_is_valid,
        mock_persistent_query_example_api_get_by_id,
    ):
        """test_authenticated_returns_http_200"""
        mock_persistent_query_example_data.return_value = {}
        mock_persistent_query_example_save.return_value = None
        mock_persistent_query_example_is_valid.return_value = True
        mock_persistent_query_example_api_get_by_id.return_value = None
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_patch(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(persistent_query_example_api, "get_by_id")
    @patch.object(PersistentQueryExampleSerializer, "is_valid")
    @patch.object(PersistentQueryExampleSerializer, "save")
    @patch.object(PersistentQueryExampleSerializer, "data")
    def test_superuser_returns_http_200(
        self,
        mock_persistent_query_example_data,
        mock_persistent_query_example_save,
        mock_persistent_query_example_is_valid,
        mock_persistent_query_example_api_get_by_id,
    ):
        """test_superuser_returns_http_200"""
        mock_persistent_query_example_data.return_value = {}
        mock_persistent_query_example_save.return_value = None
        mock_persistent_query_example_is_valid.return_value = True
        mock_persistent_query_example_api_get_by_id.return_value = None
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_patch(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestPersistentQueryExampleDetailDelete(SimpleTestCase):
    """Test Persistent Query Example Detail Delete"""

    def test_anonymous_returns_http_403(self):
        """test_anonymous_returns_http_403"""

        response = RequestMock.do_request_delete(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            AnonymousUser(),
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(persistent_query_example_api, "get_by_id")
    @patch.object(persistent_query_example_api, "delete")
    def test_authenticated_returns_http_200(
        self, persistent_query_example_api_delete, persistent_query_example_get_by_id
    ):
        """test_authenticated_returns_http_200"""

        persistent_query_example_api_delete.return_value = None
        persistent_query_example_get_by_id.return_value = None
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_delete(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    @patch.object(persistent_query_example_api, "get_by_id")
    @patch.object(persistent_query_example_api, "delete")
    def test_superuser_returns_http_200(
        self, persistent_query_example_api_delete, persistent_query_example_get_by_id
    ):
        """test_superuser_returns_http_200"""

        persistent_query_example_api_delete.return_value = None
        persistent_query_example_get_by_id.return_value = None
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_delete(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            mock_user,
            param={"pk": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)


class TestPersistentQueryExampleByNameGet(SimpleTestCase):
    """Test Persistent Query Example By Name Get"""

    @patch.object(persistent_query_example_api, "get_by_name")
    @patch.object(PersistentQueryExampleSerializer, "data")
    def test_anonymous_returns_http_403(
        self,
        mock_persistent_query_example_serializer_data,
        mock_persistent_query_example_api_get_by_name,
    ):
        """test_anonymous_returns_http_403"""
        mock_persistent_query_example_serializer_data.return_value = {}
        mock_persistent_query_example_api_get_by_name.return_value = None
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleByName.as_view(),
            AnonymousUser(),
            param={"name": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    @patch.object(persistent_query_example_api, "get_by_name")
    @patch.object(PersistentQueryExampleSerializer, "data")
    def test_authenticated_returns_http_200(
        self,
        mock_persistent_query_example_serializer_data,
        mock_persistent_query_example_api_get_by_name,
    ):
        """test_authenticated_returns_http_200"""

        mock_persistent_query_example_serializer_data.return_value = {}
        mock_persistent_query_example_api_get_by_name.return_value = None
        mock_user = create_mock_user("1")

        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleByName.as_view(),
            mock_user,
            param={"name": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)

    @patch.object(persistent_query_example_api, "get_by_name")
    @patch.object(PersistentQueryExampleSerializer, "data")
    def test_superuser_returns_http_200(
        self,
        mock_persistent_query_example_serializer_data,
        mock_persistent_query_example_api_get_by_name,
    ):
        """test_superuser_returns_http_200"""

        mock_persistent_query_example_serializer_data.return_value = {}
        mock_persistent_query_example_api_get_by_name.return_value = None
        mock_user = create_mock_user("1", is_staff=True)

        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleByName.as_view(),
            mock_user,
            param={"name": 0},
        )

        self.assertEqual(response.status_code, status.HTTP_200_OK)



"""XML utils
"""
from core_main_app.commons.exceptions import XMLError
from xml_utils.xsd_types.xsd_types import get_xsd_floating_numbers, get_xsd_numbers


def validate_element_value(element_name, element_type, element_value, namespace_prefix):
    """Validates the element

    Args:
        element_name:
        element_type:
        element_value:
        namespace_prefix:

    Returns:

    """
    error = None

    # Floating number
    if element_type in get_xsd_floating_numbers(namespace_prefix):
        try:
            float(element_value)
        except ValueError:
            error = "Element {} must be a number.".format(element_name)
    # Number
    elif element_type in get_xsd_numbers(namespace_prefix):
        try:
            int(element_value)
        except ValueError:
            error = "Element {} must be an integer.".format(element_name)

    return error


def get_enumerations(data_structure_element):
    """

    Args:
        data_structure_element:

    Returns:

    """
    # find child simple type
    try:
        while data_structure_element.tag != "simple_type":
            data_structure_element = data_structure_element.children.all()[0]
        simple_type_element = data_structure_element.children.all()[0]
    except:
        raise XMLError("Unable to find a simple type for the data structure element.")

    enums = []
    for enum_element in simple_type_element.children.all():
        if enum_element.tag == "enumeration":
            enums.append(enum_element.value)
    return enums



""" Persistent Query Example API
"""
from core_main_app.access_control.api import has_perm_administration
from core_main_app.access_control.decorators import access_control
from core_explore_common_app.access_control.api import (
    can_read_persistent_query,
    can_write_persistent_query,
)
from core_explore_example_app.components.persistent_query_example.models import (
    PersistentQueryExample,
)


@access_control(can_write_persistent_query)
def upsert(persistent_query_example, user):
    """Saves or update persistent query

    Args:
        persistent_query_example:
        user:

    Returns:

    """
    persistent_query_example.save()
    return persistent_query_example


@access_control(can_read_persistent_query)
def get_by_id(persistent_query_example_id, user):
    """Return the Persistent Query Example with the given id

    Args:
        persistent_query_example_id:
        user:

    Returns:

    """
    return PersistentQueryExample.get_by_id(persistent_query_example_id)


@access_control(can_read_persistent_query)
def get_by_name(persistent_query_example_name, user):
    """Return the Persistent Query Example with the given name

    Args:
        persistent_query_example_name:
        user:
    Returns:

    """
    return PersistentQueryExample.get_by_name(persistent_query_example_name)


@access_control(can_write_persistent_query)
def delete(persistent_query_example, user):
    """Deletes the Persistent Query Example and the element associated

    Args:
        persistent_query_example:
        user:
    """
    persistent_query_example.delete()


@access_control(can_write_persistent_query)
def set_name(persistent_query_example, name, user):
    """Set name to Persistent Query Example

    Args:
        persistent_query_example:
        name:
        user:
    """
    persistent_query_example.name = name
    persistent_query_example.save()


@access_control(has_perm_administration)
def get_all(user):
    """get all Persistent Query Example

    Args:
        user:
    """
    return PersistentQueryExample.get_all()


@access_control(can_read_persistent_query)
def get_all_by_user(user):
    """get persistent Query Example by user

    Args:
        user:
    """
    return PersistentQueryExample.get_all_by_user(user.id)



""" Discover rules for core explore example app.
"""
import logging

from core_main_app.permissions import rights as main_rights
from core_explore_example_app.permissions import rights as explore_example_rights

logger = logging.getLogger(__name__)


def init_permissions(apps):
    """Initialization of groups and permissions.

    Args:
        apps (Apps): List of applications to init
    """
    try:
        group = apps.get_model("auth", "Group")
        permission = apps.get_model("auth", "Permission")

        # Get or Create the default group
        default_group, created = group.objects.get_or_create(
            name=main_rights.DEFAULT_GROUP
        )

        # Get explore example permissions
        explore_access_perm = permission.objects.get(
            codename=explore_example_rights.EXPLORE_EXAMPLE_ACCESS
        )
        explore_save_query_perm = permission.objects.get(
            codename=explore_example_rights.EXPLORE_EXAMPLE_SAVE_QUERY
        )
        explore_delete_query_perm = permission.objects.get(
            codename=explore_example_rights.EXPLORE_EXAMPLE_DELETE_QUERY
        )
        explore_example_data_structure_access_perm = permission.objects.get(
            codename=explore_example_rights.EXPLORE_EXAMPLE_DATA_STRUCTURE_ACCESS
        )

        # Add permissions to default group
        default_group.permissions.add(
            explore_access_perm,
            explore_save_query_perm,
            explore_delete_query_perm,
            explore_example_data_structure_access_perm,
        )
    except Exception as exception:
        logger.error(
            "Impossible to init explore_example permissions: %s", str(exception)
        )



""" Conf urls
"""

from django.conf.urls import include
from django.contrib import admin
from django.urls import re_path

from core_explore_example_app import urls as core_explore_example_app_urls

urlpatterns = [
    re_path(r"^admin/", include(admin.site.urls)),
] + core_explore_example_app_urls.urlpatterns



"""Saved Query API
"""
from core_explore_example_app.components.saved_query.models import SavedQuery


def get_all():
    """Get all Saved Query

    Returns:

    """
    return SavedQuery.get_all()


def get_by_id(query_id):
    """Get a Saved Query

    Args:
        query_id:

    Returns:

    """
    return SavedQuery.get_by_id(query_id)


def get_all_by_user_and_template(user_id, template_id):
    """Gets a saved query by user id and template id

    Args:
        user_id:
        template_id:

    Returns:

    """
    return SavedQuery.get_all_by_user_and_template(user_id, template_id)


def delete(saved_query):
    """Deletes a saved query

    Args:
        saved_query:

    Returns:

    """
    saved_query.delete()


def upsert(saved_query):
    """Saves or Updates a saved query

    Args:
        saved_query:

    Returns:

    """
    saved_query.save()
    return saved_query



""" Explore data Structure api
"""

from core_explore_example_app.components.explore_data_structure.models import (
    ExploreDataStructure,
)
from core_explore_example_app.utils.parser import generate_form


def get_by_user_id_and_template_id(user_id, template_id):
    """Returns object with the given user id and template id

    Args:
        user_id:
        template_id:

    Returns:

    """
    return ExploreDataStructure.get_by_user_id_and_template_id(user_id, template_id)


def upsert(explore_data_structure):
    """Saves explore data structure

    Args:
        explore_data_structure:

    Returns:

    """
    explore_data_structure.save()
    return explore_data_structure


def create_and_get_explore_data_structure(template, request):
    """Get Data structure from template and user, generate them if no exist

    Args:
        template:
        request:

    Returns: Explore Data structure

    """
    try:
        # get data structure
        explore_data_structure = get_by_user_id_and_template_id(
            user_id=str(request.user.id), template_id=template.id
        )
    except:
        # create explore data structure
        explore_data_structure = ExploreDataStructure(
            user=str(request.user.id),
            template=template,
            name=template.filename,
        )
        upsert(explore_data_structure)
        # generate the root element
        root_element = generate_form(
            template.content, data_structure=explore_data_structure, request=request
        )
        explore_data_structure.data_structure_element_root = root_element
        upsert(explore_data_structure)

    # Return the data structure
    return explore_data_structure


def get_by_id(explore_data_structure_id):
    """Return the explore data structure with the given id

    Args:
        explore_data_structure_id:

    Returns:

    """
    return ExploreDataStructure.get_by_id(explore_data_structure_id)



"""Explore Data Structure
"""
from django.core.exceptions import ObjectDoesNotExist
from django.db import models

from core_main_app.commons import exceptions as exceptions
from core_parser_app.components.data_structure.models import DataStructure
from core_explore_example_app.permissions import rights


class ExploreDataStructure(DataStructure):
    """Explore data structure"""

    selected_fields_html_tree = models.TextField(blank=True, default=None, null=True)

    @staticmethod
    def get_permission():
        """Return permission

        Returns:

        """
        return f"{rights.EXPLORE_EXAMPLE_CONTENT_TYPE}.{rights.EXPLORE_EXAMPLE_DATA_STRUCTURE_ACCESS}"

    @staticmethod
    def get_by_user_id_and_template_id(user_id, template_id):
        """Return explore data structure given its id and template id

        Returns:

        """
        try:
            return ExploreDataStructure.objects.get(
                user=str(user_id), template=str(template_id)
            )
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_by_id(data_structure_id):
        """Return the object with the given id.

        Args:
            data_structure_id:

        Returns:
            Explore Data Structure (obj): ExploreDataStructure object with the given id

        """
        try:
            return ExploreDataStructure.objects.get(pk=str(data_structure_id))
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))



""" Integration Test for Persistent Query Example Rest API
"""

from django.contrib.auth.models import AnonymousUser
from rest_framework import status

from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_explore_example_app.rest.persistent_query_example import (
    views as persistent_query_example_views,
)
from tests.components.persistent_query_example.fixtures.fixtures import (
    PersistentQueryExampleFixtures,
)

fixture_data_structure = PersistentQueryExampleFixtures()


class TestPersistentQueryExampleListAdmin(MongoIntegrationBaseTestCase):
    """Test Persistent Query Example List Admin"""

    fixture = fixture_data_structure

    def setUp(self):
        """setUp"""

        super().setUp()

        self.user = create_mock_user("1", is_staff=True, is_superuser=True)

        self.data = {
            "user_id": "1",
            "name": "persistent_query_name",
        }

    def test_get_returns_all_user_persistent_query_example(self):
        """test_get_returns_all_user_persistent_query_example"""

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.AdminPersistentQueryExampleList.as_view(),
            self.user,
        )

        # Assert
        self.assertEqual(len(response.data), 3)

    def test_post_returns_http_201(self):
        """test_post_returns_http_201"""

        # Arrange

        # Act
        response = RequestMock.do_request_post(
            persistent_query_example_views.AdminPersistentQueryExampleList.as_view(),
            self.user,
            data=self.data,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestPersistentQueryExampleList(MongoIntegrationBaseTestCase):
    """Test Persistent Query Example List"""

    fixture = fixture_data_structure

    def setUp(self):
        """setUp"""

        super().setUp()

        self.user = create_mock_user("1")

        self.data = {
            "name": "persistent_query_name",
        }

    def test_get_returns_all_persistent_query_example(self):
        """test_get_returns_all_persistent_query_example"""

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleList.as_view(),
            self.user,
        )

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_post_returns_http_201(self):
        """test_post_returns_http_201"""

        # Act
        response = RequestMock.do_request_post(
            persistent_query_example_views.PersistentQueryExampleList.as_view(),
            self.user,
            data=self.data,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)


class TestPersistentQueryExampleDetail(MongoIntegrationBaseTestCase):
    """Test Persistent Query Example Detail"""

    fixture = fixture_data_structure

    def setUp(self):
        """setUp"""

        super().setUp()

    def test_get_returns_http_200(self):
        """test_get_returns_http_200"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            user,
            param={"pk": str(self.fixture.persistent_query_example_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_returns_persistent_query_example(self):
        """test_get_returns_persistent_query_example"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            user,
            param={"pk": str(self.fixture.persistent_query_example_1.id)},
        )

        # Assert
        self.assertEqual(
            response.data["name"], self.fixture.persistent_query_example_1.name
        )

    def test_get_other_user_persistent_query_example_returns_persistent_query_example(
        self,
    ):
        """test_get_other_user_persistent_query_example_returns_persistent_query_example"""

        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            user,
            param={"pk": str(self.fixture.persistent_query_example_1.id)},
        )

        # Assert
        self.assertEqual(
            response.data["name"], self.fixture.persistent_query_example_1.name
        )

    def test_get_other_user_persistent_query_example_as_anonymous_user(self):
        """test_get_other_user_persistent_query_example_as_anonymous_user"""

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            AnonymousUser(),
            param={"pk": str(self.fixture.persistent_query_example_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_get_wrong_id_returns_http_404(self):
        """test_get_wrong_id_returns_http_404"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            user,
            param={"pk": -1},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_delete_returns_http_204(self):
        """test_delete_returns_http_204"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            user,
            param={"pk": str(self.fixture.persistent_query_example_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    def test_delete_other_user_persistent_query_example_returns_http_403(self):
        """test_delete_other_user_persistent_query_example_returns_http_403"""

        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_delete(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            user,
            param={"pk": str(self.fixture.persistent_query_example_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_delete_wrong_id_returns_http_404(self):
        """test_delete_wrong_id_returns_http_404"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            user,
            param={"pk": -1},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_other_user_persistent_query_example_returns_http_403(self):
        """test_patch_other_user_persistent_query_example_returns_http_403"""

        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            user,
            param={"pk": str(self.fixture.persistent_query_example_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_patch_returns_updated_name(self):
        """test_patch_returns_updated_name"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            user,
            param={"pk": str(self.fixture.persistent_query_example_1.id)},
            data={"name": "new_name"},
        )

        # Assert
        self.assertEqual(response.data["name"], "new_name")

    def test_patch_wrong_id_returns_http_404(self):
        """test_patch_wrong_id_returns_http_404"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            persistent_query_example_views.PersistentQueryExampleDetail.as_view(),
            user,
            param={"pk": -1},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


class TestPersistentQueryExampleByName(MongoIntegrationBaseTestCase):
    """Test Persistent Query Example By Name"""

    fixture = fixture_data_structure

    def setUp(self):
        """setUp"""

        super().setUp()

    def test_get_returns_http_200(self):
        """test_get_returns_http_200"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleByName.as_view(),
            user,
            param={"name": str(self.fixture.persistent_query_example_1.name)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_by_name_returns_persistent_query_example(self):
        """test_get_by_name_returns_persistent_query_example"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleByName.as_view(),
            user,
            param={"name": str(self.fixture.persistent_query_example_1.name)},
        )

        # Assert
        self.assertEqual(
            response.data["name"], self.fixture.persistent_query_example_1.name
        )

    def test_get_other_user_persistent_query_example_by_name_returns_persistent_query_example(
        self,
    ):
        """test_get_other_user_persistent_query_example_by_name_returns_persistent_query_example"""

        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleByName.as_view(),
            user,
            param={"name": str(self.fixture.persistent_query_example_1.name)},
        )

        # Assert
        self.assertEqual(
            response.data["name"], self.fixture.persistent_query_example_1.name
        )

    def test_get_other_user_persistent_query_example_by_name_as_anonymous_user(self):
        """test_get_other_user_persistent_query_example_by_name_as_anonymous_user"""

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleByName.as_view(),
            AnonymousUser(),
            param={"name": str(self.fixture.persistent_query_example_1.name)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_get_wrong_name_returns_http_404(self):
        """test_get_wrong_name_returns_http_404"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            persistent_query_example_views.PersistentQueryExampleByName.as_view(),
            user,
            param={"name": ""},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)



""" Persistent Query Example model
"""
from django.core.exceptions import ObjectDoesNotExist

from core_explore_common_app.components.abstract_persistent_query.models import (
    AbstractPersistentQuery,
)
from core_main_app.commons import exceptions


class PersistentQueryExample(AbstractPersistentQuery):
    """Persistent Query Example"""

    class Meta:
        """Meta"""

        verbose_name = "Persistent Query by Example"
        verbose_name_plural = "Persistent Queries by Example"

    @staticmethod
    def get_by_id(query_id):
        """Get a persistent query example

        Args:
            query_id:

        Returns:

        """
        try:
            return PersistentQueryExample.objects.get(pk=query_id)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_by_name(query_name):
        """Get a persistent query example

        Args:
            query_name:

        Returns:

        """
        try:
            return PersistentQueryExample.objects.get(name=query_name)
        except ObjectDoesNotExist as exception:
            raise exceptions.DoesNotExist(str(exception))
        except Exception as exception:
            raise exceptions.ModelError(str(exception))

    @staticmethod
    def get_all():
        """Return all persistent query example.

        Returns:

        """
        return PersistentQueryExample.objects.all()

    @staticmethod
    def get_all_by_user(user_id):
        """Return all persistent query example by user.


        Args:
            user_id:

        Returns:

        """
        return PersistentQueryExample.objects.filter(user_id=str(user_id))

    @staticmethod
    def get_none():
        """Return None object, used by data.

        Returns:

        """
        return PersistentQueryExample.objects.none()



"""Util to build user readable queries
"""

from core_parser_app.components.data_structure_element import (
    api as data_structure_element_api,
)
from core_explore_example_app.utils.mongo_query import get_parent_name
from core_explore_example_app.utils.query_builder import (
    get_element_value,
    get_element_comparison,
)


def build_pretty_criteria(element_name, comparison, value, is_not=False):
    """Returns a pretty representation of the criteria

    Args:
        element_name:
        comparison:
        value:
        is_not:

    Returns:

    """
    pretty_criteria = ""

    if is_not:
        pretty_criteria += "NOT("

    pretty_criteria += element_name

    if comparison == "lt":
        pretty_criteria += " &lt; "
    elif comparison == "lte":
        pretty_criteria += " &le; "
    elif comparison == "=":
        pretty_criteria += "="
    elif comparison == "gte":
        pretty_criteria += " &ge; "
    elif comparison == "gt":
        pretty_criteria += " &gt; "
    elif comparison == "is":
        pretty_criteria += " is "
    elif comparison == "like":
        pretty_criteria += " like "

    if value:
        pretty_criteria += value
    else:
        pretty_criteria += " &ldquo;  &ldquo;"

    if is_not:
        pretty_criteria += ")"

    return pretty_criteria


def build_query_pretty_criteria(query_value, is_not):
    """Returns a pretty representation of the query

    Args:
        query_value:
        is_not:

    Returns:

    """
    if is_not:
        return "NOT(" + query_value + ")"

    return query_value


def build_or_pretty_criteria(query, criteria):
    """Returns a pretty representation of the OR

    Args:
        query:
        criteria:

    Returns:

    """
    return "(" + query + " OR " + criteria + ")"


def build_and_pretty_criteria(query, criteria):
    """Returns a pretty representation of the AND

    Args:
        query:
        criteria:

    Returns:

    """
    return "(" + query + " AND " + criteria + ")"


def fields_to_pretty_query(form_values):
    """Transforms fields from the HTML form into pretty representation

    Args:
        form_values:

    Returns:

    """

    query = ""

    for field in form_values:
        bool_comp = field["operator"]
        if bool_comp == "NOT":
            is_not = True
        else:
            is_not = False

        # get element value
        value = get_element_value(field)
        # get comparison operator
        comparison = get_element_comparison(field)

        element_type = field.get("type", None)
        if element_type == "query":
            criteria = build_query_pretty_criteria(field["name"], is_not)
        else:
            criteria = build_pretty_criteria(field["name"], comparison, value, is_not)

        if bool_comp == "OR":
            query = build_or_pretty_criteria(query, criteria)
        elif bool_comp == "AND":
            query = build_and_pretty_criteria(query, criteria)
        else:
            if form_values.index(field) == 0:
                query += criteria
            else:
                query = build_and_pretty_criteria(query, criteria)

    return query


def sub_elements_to_pretty_query(form_values, namespaces, request):
    """Transforms HTML fields in a user readable query

    Args:
        form_values:
        namespaces:
        request:

    Returns:

    """
    # get the parent path using the first element of the list
    parent_name = get_parent_name(form_values[0]["id"], namespaces, request)

    list_criteria = []
    for i in range(0, len(form_values)):
        field = form_values[i]
        if field["selected"] is True:
            bool_comp = field["operator"]
            if bool_comp == "NOT":
                is_not = True
            else:
                is_not = False

            data_structure_element = data_structure_element_api.get_by_id(
                field["id"], request
            )
            element_name = data_structure_element.options["name"]
            value = get_element_value(field)
            comparison = get_element_comparison(field)

            criteria = build_pretty_criteria(element_name, comparison, value, is_not)

            list_criteria.append(criteria)

    query = "{0}({1})".format(parent_name, ", ".join(list_criteria))

    return query



""" Core Explore Example exceptions
"""


class MongoQueryException(Exception):
    """
    Exception raised by mongo query builder
    """

    def __init__(self, message):
        self.message = message



""" Add Explore Example in main menu
"""

from django.urls import reverse
from menu import Menu, MenuItem

from core_explore_example_app.settings import EXPLORE_EXAMPLE_MENU_NAME

# FIXME: CHECK AUTHENTICATION !
Menu.add_item(
    "main", MenuItem(EXPLORE_EXAMPLE_MENU_NAME, reverse("core_explore_example_index"))
)



""" Settings for core_explore_example_app

Settings with the following syntax can be overwritten at the project level:
SETTING_NAME = getattr(settings, "SETTING_NAME", "Default Value")
"""
from django.conf import settings

if not settings.configured:
    settings.configure()

INSTALLED_APPS = getattr(settings, "INSTALLED_APPS", [])

PARSER_DOWNLOAD_DEPENDENCIES = getattr(settings, "PARSER_DOWNLOAD_DEPENDENCIES", False)
QUERIES_MAX_DAYS_IN_DATABASE = getattr(settings, "QUERIES_MAX_DAYS_IN_DATABASE", 7)
EXPLORE_EXAMPLE_MENU_NAME = getattr(
    settings, "EXPLORE_EXAMPLE_MENU_NAME", "Query by Example"
)



"""Integration tests for Saved Query rest api
"""

from rest_framework import status

from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
import core_explore_example_app.rest.saved_query.views as saved_query_views
from tests.rest.saved_query.fixtures.fixtures import SavedQueryFixtures

fixture_data = SavedQueryFixtures()


class TestGetSavedQueryList(MongoIntegrationBaseTestCase):
    """Test Get Saved Query List"""

    fixture = fixture_data

    def setUp(self):
        """setUp"""

        super().setUp()

    def test_get_all_returns_status_200_with_no_permission_needed(self):
        """test_get_all_returns_status_200_with_no_permission_needed"""

        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            saved_query_views.SavedQueryList.as_view(), user
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestGetSavedQueryDetail(MongoIntegrationBaseTestCase):
    """Test Get Saved Query Detail"""

    fixture = fixture_data

    def setUp(self):
        """setUp"""

        super().setUp()
        self.data = None

    def test_get_returns_object_when_found(self):
        """test_get_returns_object_when_found"""

        # Arrange
        user = create_mock_user("0")
        self.param = {"pk": self.fixture.data_1.id}

        # Act
        response = RequestMock.do_request_get(
            saved_query_views.SavedQueryDetail.as_view(), user, self.data, self.param
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_raise_404_when_not_found(self):
        """test_get_raise_404_when_not_found"""

        # Arrange
        user = create_mock_user("0")
        self.param = {"pk": -1}

        # Act
        response = RequestMock.do_request_get(
            saved_query_views.SavedQueryDetail.as_view(), user, self.data, self.param
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_get_raise_500_sever_error_when_general_error_occurred(self):
        """test_get_raise_500_sever_error_when_general_error_occurred"""

        # Arrange
        user = create_mock_user("0")
        self.param = {"pk": "test"}

        # Act
        response = RequestMock.do_request_get(
            saved_query_views.SavedQueryDetail.as_view(), user, self.data, self.param
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_500_INTERNAL_SERVER_ERROR)


class TestDeleteSavedQueryDetail(MongoIntegrationBaseTestCase):
    """Test Delete Saved Query Detail"""

    fixture = fixture_data

    def setUp(self):
        """setUp"""

        super(TestDeleteSavedQueryDetail, self).setUp()
        self.data = None

    def test_delete_returns_403_status_when_user_is_unhautorized(self):
        """test_delete_returns_403_status_when_user_is_unhautorized"""

        # Arrange
        user = create_mock_user("0")
        self.param = {"pk": self.fixture.data_1.id}

        # Act
        response = RequestMock.do_request_delete(
            saved_query_views.SavedQueryDetail.as_view(), user, self.data, self.param
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_delete_returns_204_status_when_found(self):
        """test_delete_returns_204_status_when_found"""

        # Arrange
        user = create_mock_user("1")
        self.param = {"pk": self.fixture.data_1.id}

        # Act
        response = RequestMock.do_request_delete(
            saved_query_views.SavedQueryDetail.as_view(), user, self.data, self.param
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    def test_delete_raise_404_when_not_found(self):
        """test_delete_raise_404_when_not_found"""

        # Arrange
        user = create_mock_user("0")
        self.param = {"pk": -1}

        # Act
        response = RequestMock.do_request_delete(
            saved_query_views.SavedQueryDetail.as_view(), user, self.data, self.param
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)



""" Unit Test Explore Data Structure
"""
from unittest.case import TestCase

from mock import patch

from core_main_app.components.template.models import Template
from core_parser_app.components.data_structure.models import DataStructureElement
from core_explore_example_app.components.explore_data_structure import (
    api as explore_data_structure_api,
)
from core_explore_example_app.components.explore_data_structure.models import (
    ExploreDataStructure,
)


class TestExploreDataStructureInsert(TestCase):
    """Test Explore Data Structure Insert"""

    @patch.object(ExploreDataStructure, "get_by_user_id_and_template_id")
    @patch.object(ExploreDataStructure, "save")
    def test_explore_data_structure_upsert_single_structure(self, mock_save, mock_list):
        """test_explore_data_structure_upsert_single_structure"""

        data = create_explore_data_structure("1", "name_title_1")
        mock_save.return_value = data
        mock_list.return_value = None
        explore_data_structure_api.upsert(data)


class TestExploreDataStructureGetByUserIdAndTemplateId(TestCase):
    """Test Explore Data Structure Get By User Id And TemplateId"""

    @patch.object(ExploreDataStructure, "get_by_user_id_and_template_id")
    def test_explore_data_structure_get_by_user_and_template_returns_explore_data_structure(
        self, mock_list
    ):
        """test_explore_data_structure_get_by_user_and_template_returns_explore_data_structure"""

        # Arrange
        mock_data = create_explore_data_structure(user="1", name="name_title_1")
        mock_list.return_value = mock_data
        # Act
        result = explore_data_structure_api.get_by_user_id_and_template_id("1", "1")
        # Assert
        self.assertTrue(isinstance(result, ExploreDataStructure))


def _get_template():
    """Returns a template

    Returns:

    """
    template = Template()
    template.id_field = 1
    xsd = (
        '<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">'
        '<xs:element name="tag"></xs:element></xs:schema>'
    )
    template.content = xsd
    return template


def _get_data_structure_element(user, data_structure):
    """Return a data structure element

    Args:
        user:
        data_structure:

    Returns:

    """
    data_structure_element = DataStructureElement(
        user=user,
        tag="tag",
        value="value",
        options={},
        data_structure=data_structure,
    )
    data_structure_element.save()
    return data_structure_element


def create_explore_data_structure(user, name):
    """Creates Explore Data Structure

    Args:
        user:
        name:

    Returns:

    """
    template = _get_template()
    explore_data_structure = ExploreDataStructure(
        user=user, template=template, name=name, data_structure_element_root=None
    )
    return explore_data_structure



""" Unit Test Persistent Query Example
"""

from django.contrib.auth.models import AnonymousUser


from core_main_app.access_control.exceptions import AccessControlError
from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_explore_common_app.settings import CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT
import core_explore_example_app.components.persistent_query_example.api as persistent_query_example_api
from core_explore_example_app.components.persistent_query_example.models import (
    PersistentQueryExample,
)
from tests.components.persistent_query_example.fixtures.fixtures import (
    PersistentQueryExampleFixtures,
)

fixture_persistent_query_example = PersistentQueryExampleFixtures()


class TestPersistentQueryExampleGetById(MongoIntegrationBaseTestCase):
    """Test Persistent Query Example Get By Id"""

    fixture = fixture_persistent_query_example

    def test_get_by_id_as_superuser_returns_persistent_query_example(self):
        """test_get_by_id_as_superuser_returns_persistent_query_example"""

        # Arrange
        persistent_query_example_id = self.fixture.persistent_query_example_1.id
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        persistent_query_example = persistent_query_example_api.get_by_id(
            persistent_query_example_id, mock_user
        )

        # Assert
        self.assertTrue(isinstance(persistent_query_example, PersistentQueryExample))

    def test_get_by_id_as_owner_returns_persistent_query_example(self):
        """test_get_by_id_as_owner_returns_persistent_query_example"""

        # Arrange
        persistent_query_example_id = self.fixture.persistent_query_example_1.id
        mock_user = create_mock_user("1")

        # Act
        persistent_query_example = persistent_query_example_api.get_by_id(
            persistent_query_example_id, mock_user
        )

        # Assert
        self.assertTrue(isinstance(persistent_query_example, PersistentQueryExample))

    def test_get_by_id_as_user_not_owner_returns_persistent_query_example(self):
        """test_get_by_id_as_user_not_owner_returns_persistent_query_example"""

        # Arrange
        persistent_query_example_id = self.fixture.persistent_query_example_1.id
        mock_user = create_mock_user("0")

        # Act # Assert
        self.assertTrue(
            isinstance(
                persistent_query_example_api.get_by_id(
                    persistent_query_example_id, mock_user
                ),
                PersistentQueryExample,
            )
        )

    def test_get_by_id_as_anonymous_user(self):
        """test_get_by_id_as_anonymous_user"""

        # Arrange
        persistent_query_example_id = self.fixture.persistent_query_example_1.id

        # Act # Assert
        if CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            self.assertTrue(
                isinstance(
                    persistent_query_example_api.get_by_id(
                        persistent_query_example_id, AnonymousUser()
                    ),
                    PersistentQueryExample,
                )
            )

        else:
            with self.assertRaises(AccessControlError):
                persistent_query_example_api.get_by_id(
                    persistent_query_example_id, AnonymousUser()
                )


class TestPersistentQueryExampleGetByName(MongoIntegrationBaseTestCase):
    """Test Persistent Query Example Get By Name"""

    fixture = fixture_persistent_query_example

    def test_get_by_name_as_superuser_returns_persistent_query_example(self):
        """test_get_by_name_as_superuser_returns_persistent_query_example"""

        # Arrange
        persistent_query_example_name = self.fixture.persistent_query_example_1.name
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        persistent_query_example = persistent_query_example_api.get_by_name(
            persistent_query_example_name, mock_user
        )

        # Assert
        self.assertTrue(isinstance(persistent_query_example, PersistentQueryExample))

    def test_get_by_id_as_owner_returns_persistent_query_example(self):
        """test_get_by_id_as_owner_returns_persistent_query_example"""

        # Arrange
        persistent_query_example_name = self.fixture.persistent_query_example_1.name
        mock_user = create_mock_user("1")

        # Act
        persistent_query_example = persistent_query_example_api.get_by_name(
            persistent_query_example_name, mock_user
        )

        # Assert
        self.assertTrue(isinstance(persistent_query_example, PersistentQueryExample))

    def test_get_by_id_as_user_not_owner_returns_persistent_query_example(self):
        """test_get_by_id_as_user_not_owner_returns_persistent_query_example"""

        # Arrange
        persistent_query_example_name = self.fixture.persistent_query_example_1.name
        mock_user = create_mock_user("0")

        # Act # Assert
        self.assertTrue(
            isinstance(
                persistent_query_example_api.get_by_name(
                    persistent_query_example_name, mock_user
                ),
                PersistentQueryExample,
            )
        )

    def test_get_by_id_as_anonymous_user(self):
        """test_get_by_id_as_anonymous_user"""

        # Arrange
        persistent_query_example_name = self.fixture.persistent_query_example_1.name

        # Act # Assert
        if CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            self.assertTrue(
                isinstance(
                    persistent_query_example_api.get_by_name(
                        persistent_query_example_name, AnonymousUser()
                    ),
                    PersistentQueryExample,
                )
            )

        else:
            with self.assertRaises(AccessControlError):
                persistent_query_example_api.get_by_name(
                    persistent_query_example_name, AnonymousUser()
                )


class TestPersistentQueryExampleDelete(MongoIntegrationBaseTestCase):
    """Test Persistent Query Example Delete"""

    fixture = fixture_persistent_query_example

    def test_delete_others_persistent_query_example_as_superuser_deletes_persistent_query_example(
        self,
    ):
        """test_delete_others_persistent_query_example_as_superuser_deletes_persistent_query_example"""

        # Arrange
        persistent_query_example = self.fixture.persistent_query_example_1
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        persistent_query_example_api.delete(persistent_query_example, mock_user)

    def test_delete_own_persistent_query_example_deletes_persistent_query_example(self):
        """test_delete_own_persistent_query_example_deletes_persistent_query_example"""

        # Arrange
        persistent_query_example = self.fixture.persistent_query_example_1
        mock_user = create_mock_user("1")

        # Act
        persistent_query_example_api.delete(persistent_query_example, mock_user)

    def test_delete_others_persistent_query_example_raises_error(self):
        """test_delete_others_persistent_query_example_raises_error"""

        # Arrange
        persistent_query_example = self.fixture.persistent_query_example_1
        mock_user = create_mock_user("0")

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_example_api.delete(persistent_query_example, mock_user)

    def test_delete_others_persistent_query_example_as_anonymous_raises_error(
        self,
    ):
        """test_delete_others_persistent_query_example_as_anonymous_raises_error"""

        # Arrange
        persistent_query_example = self.fixture.persistent_query_example_1

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_example_api.delete(
                persistent_query_example, AnonymousUser()
            )


class TestPersistentQueryExampleUpdate(MongoIntegrationBaseTestCase):
    """Test Persistent Query Example Update"""

    fixture = fixture_persistent_query_example

    def test_update_others_persistent_query_example_as_superuser_updates_data_structure(
        self,
    ):
        """test_update_others_persistent_query_example_as_superuser_updates_data_structure"""

        # Arrange
        persistent_query_example = self.fixture.persistent_query_example_1
        persistent_query_example.name = "new_name_persistent_query_example_1"
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)
        # Act
        result = persistent_query_example_api.upsert(
            persistent_query_example, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryExample))
        self.assertTrue(result.name, "new_name_persistent_query_example_1")

    def test_update_own_persistent_query_example_updates_persistent_query_example(self):
        """test_update_own_persistent_query_example_updates_persistent_query_example"""

        # Arrange
        persistent_query_example = self.fixture.persistent_query_example_1
        mock_user = create_mock_user("1")
        persistent_query_example.name = "new_name_persistent_query_example_1"
        # Act
        result = persistent_query_example_api.upsert(
            persistent_query_example, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryExample))
        self.assertTrue(result.name, "new_name_persistent_query_example_1")

    def test_update_others_persistent_query_example_raises_error(self):
        """test_update_others_persistent_query_example_raises_error"""

        # Arrange
        persistent_query_example = self.fixture.persistent_query_example_1
        persistent_query_example.name = "new_name_persistent_query_example_1"
        mock_user = create_mock_user("0")

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_example_api.upsert(persistent_query_example, mock_user)

    def test_update_others_persistent_query_example_as_anonymous_raises_error(
        self,
    ):
        """test_update_others_persistent_query_example_as_anonymous_raises_error"""
        # Arrange
        persistent_query_example = self.fixture.persistent_query_example_1

        # Act # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_example_api.upsert(
                persistent_query_example, AnonymousUser()
            )


class TestPersistentQueryExampleCreate(MongoIntegrationBaseTestCase):
    """Test Persistent Query Example Create"""

    fixture = fixture_persistent_query_example

    def test_create_others_persistent_query_example_as_superuser_creates_persistent_query_example(
        self,
    ):
        """test_create_others_persistent_query_example_as_superuser_creates_persistent_query_example"""
        # Arrange
        persistent_query_example = PersistentQueryExample(
            name="new_persistent_query_example", user_id="0"
        )
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)
        # Act
        result = persistent_query_example_api.upsert(
            persistent_query_example, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryExample))
        self.assertTrue(result.name, "new_persistent_query_example")

    def test_create_persistent_query_example_as_user_creates_persistent_query_example(
        self,
    ):
        """test_create_persistent_query_example_as_user_creates_persistent_query_example"""

        # Arrange
        persistent_query_example = PersistentQueryExample(
            name="new_persistent_query_example", user_id="1"
        )
        mock_user = create_mock_user("1")

        # Act
        result = persistent_query_example_api.upsert(
            persistent_query_example, mock_user
        )
        # Assert
        self.assertTrue(isinstance(result, PersistentQueryExample))
        self.assertTrue(result.name, "new_persistent_query_example")

    def test_create_persistent_query_example_as_anonymous_user(self):
        """test_create_persistent_query_example_as_anonymous_user"""

        # Arrange
        persistent_query_example = PersistentQueryExample(
            name="new_persistent_query_example", user_id="None"
        )

        # Act
        if CAN_ANONYMOUS_ACCESS_PUBLIC_DOCUMENT:
            result = persistent_query_example_api.upsert(
                persistent_query_example, AnonymousUser()
            )
            # Assert
            self.assertTrue(isinstance(result, PersistentQueryExample))
            self.assertTrue(result.name, "new_persistent_query_example")

        else:
            with self.assertRaises(AccessControlError):
                persistent_query_example_api.upsert(
                    persistent_query_example, AnonymousUser()
                )


class TestPersistentQueryExampleGetAll(MongoIntegrationBaseTestCase):
    """Test Persistent Query Example Get All"""

    fixture = fixture_persistent_query_example

    def test_get_all_as_superuser_returns_all_persistent_query_example(self):
        """test_get_all_as_superuser_returns_all_persistent_query_example"""

        # Arrange
        mock_user = create_mock_user("0", is_staff=True, is_superuser=True)

        # Act
        result = persistent_query_example_api.get_all(mock_user)

        # Assert
        self.assertTrue(len(result), 3)

    def test_get_all_as_user_raises_error(self):
        """test_get_all_as_user_raises_error"""

        # Arrange
        mock_user = create_mock_user("1")

        # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_example_api.get_all(mock_user)

    def test_get_all_as_anonymous_user_raises_error(self):
        """test_get_all_as_anonymous_user_raises_error"""

        # Assert
        with self.assertRaises(AccessControlError):
            persistent_query_example_api.get_all(AnonymousUser())


class TestPersistentQueryExampleGetAllByUser(MongoIntegrationBaseTestCase):
    """Test Persistent Query Example Get All By User"""

    fixture = fixture_persistent_query_example

    def test_get_all_by_user_as_superuser_returns_all_user_persistent_query_example(
        self,
    ):
        """test_get_all_by_user_as_superuser_returns_all_user_persistent_query_example"""

        # Arrange
        mock_user = create_mock_user("1", is_staff=True, is_superuser=True)

        # Act
        result = persistent_query_example_api.get_all_by_user(mock_user)

        # Assert
        self.assertTrue(len(result), 1)

    def test_get_all_by_user_returns_all_user_persistent_query_example(self):
        """test_get_all_by_user_returns_all_user_persistent_query_example"""

        # Arrange
        mock_user = create_mock_user("1")

        # Act
        result = persistent_query_example_api.get_all_by_user(mock_user)

        # Assert
        self.assertTrue(len(result), 1)



""" Build Criteria Regression Test
"""
from unittest.case import TestCase

from core_explore_example_app.utils.mongo_query import (
    build_or_criteria,
    build_and_criteria,
    build_wildcard_elem_match_criteria,
)

mock_criteria_1 = {"root.variable": "value"}
mock_criteria_2 = {"root.variable.#text": "value"}
mock_criteria_3 = {"root.variable1": "value"}
mock_criteria_4 = {"root.variable1.#text": "value"}


class TestBuildOrCriteria(TestCase):
    """Test Build Or Criteria"""

    def test_build_or_criteria_regression_with_two_params(self):
        """test_build_or_criteria_regression_with_two_params"""

        # get the result from the old version which allow 2 params
        build_or_criteria_with_two_params_result = (
            self._build_or_criteria_with_two_params(mock_criteria_1, mock_criteria_2)
        )
        # get the result from the new version which allow N params
        build_or_criteria_result = build_or_criteria(mock_criteria_1, mock_criteria_2)

        self.assertDictEqual(
            build_or_criteria_result, build_or_criteria_with_two_params_result
        )

    def test_build_or_criteria_with_one_params(self):
        """test_build_or_criteria_with_one_params"""

        build_or_criteria_result = build_or_criteria(*[mock_criteria_1])

        self.assertDictEqual(build_or_criteria_result, {"$or": [mock_criteria_1]})

    def test_build_or_criteria_with_four_params(self):
        """test_build_or_criteria_with_four_params"""

        build_or_criteria_result = build_or_criteria(
            *[
                mock_criteria_1,
                mock_criteria_2,
                mock_criteria_3,
                mock_criteria_4,
            ]
        )

        self.assertDictEqual(
            build_or_criteria_result,
            {
                "$or": [
                    mock_criteria_1,
                    mock_criteria_2,
                    mock_criteria_3,
                    mock_criteria_4,
                ]
            },
        )

    def _build_or_criteria_with_two_params(self, criteria1, criteria2):
        """Builds a criteria that is the result of criteria1 or criteria2

        Args:
            criteria1:
            criteria2:

        Returns:

        """
        or_criteria = dict()
        or_criteria["$or"] = []
        or_criteria["$or"].append(criteria1)
        or_criteria["$or"].append(criteria2)
        return or_criteria


class TestBuildAndCriteria(TestCase):
    """Test Build And Criteria"""

    def test_build_and_criteria_regression_with_one_params(self):
        """test_build_and_criteria_regression_with_one_params"""

        # get the result from the old version which allow 2 params
        build_and_criteria_with_two_params_result = (
            self._build_and_criteria_with_two_params(mock_criteria_1, mock_criteria_2)
        )
        # get the result from the new version which allow N params
        build_and_criteria_result = build_and_criteria(mock_criteria_1, mock_criteria_2)

        self.assertDictEqual(
            build_and_criteria_result, build_and_criteria_with_two_params_result
        )

    def test_build_and_criteria_with_one_params(self):
        """test_build_and_criteria_with_one_params"""

        build_and_criteria_result = build_and_criteria(*[mock_criteria_1])

        self.assertDictEqual(build_and_criteria_result, {"$and": [mock_criteria_1]})

    def test_build_and_criteria_with_four_params(self):
        """test_build_and_criteria_with_four_params"""

        build_and_criteria_result = build_and_criteria(
            *[
                mock_criteria_1,
                mock_criteria_2,
                mock_criteria_3,
                mock_criteria_4,
            ]
        )

        self.assertDictEqual(
            build_and_criteria_result,
            {
                "$and": [
                    mock_criteria_1,
                    mock_criteria_2,
                    mock_criteria_3,
                    mock_criteria_4,
                ]
            },
        )

    def _build_and_criteria_with_two_params(self, criteria1, criteria2):
        """Builds a criteria that is the result of criteria1 and criteria2

        Args:
            criteria1:
            criteria2:

        Returns:

        """
        and_criteria = dict()
        and_criteria["$and"] = []
        and_criteria["$and"].append(criteria1)
        and_criteria["$and"].append(criteria2)
        return and_criteria


class TestBuildWildcardCriteria(TestCase):
    """Test Build Wildcard Criteria"""

    def test_build_wildcard_criteria_regression_with_one_params(self):
        """test_build_wildcard_criteria_regression_with_one_params"""

        # get the result from the old version which allow 2 params
        build_wildcard_criteria_with_two_params_result = (
            self._build_wildcard_elem_match_criteria(mock_criteria_1, mock_criteria_2)
        )
        # get the result from the new version which allow N params
        build_wildcard_criteria_result = build_wildcard_elem_match_criteria(
            mock_criteria_1, mock_criteria_2
        )

        self.assertDictEqual(
            build_wildcard_criteria_result,
            build_wildcard_criteria_with_two_params_result,
        )

    def test_build_wildcard_criteria_with_one_params(self):
        """test_build_wildcard_criteria_with_one_params"""

        build_wildcard_criteria_result = build_wildcard_elem_match_criteria(
            *[mock_criteria_1]
        )

        self.assertDictEqual(
            build_wildcard_criteria_result,
            {"list_content": {"$elemMatch": {"root.variable": "value"}}},
        )

    def test_build_and_criteria_with_four_params(self):
        """test_build_and_criteria_with_four_params"""

        build_wildcard_criteria_result = build_wildcard_elem_match_criteria(
            *[mock_criteria_1, mock_criteria_2, mock_criteria_3, mock_criteria_4]
        )

        self.assertDictEqual(
            build_wildcard_criteria_result,
            {
                "list_content": {
                    "$elemMatch": {
                        "root.variable": "value",
                        "root.variable.#text": "value",
                        "root.variable1": "value",
                        "root.variable1.#text": "value",
                    }
                }
            },
        )

    def _build_wildcard_elem_match_criteria(self, criteria1, criteria2):
        """Builds a criteria that is the result of criteria1 elemMatch criteria2

        Args:
            criteria1:
            criteria2:

        Returns:

        """
        elem_match_criteria = dict()
        elem_match_criteria["$elemMatch"] = {}
        elem_match_criteria["$elemMatch"].update(criteria1)
        elem_match_criteria["$elemMatch"].update(criteria2)
        return {"list_content": elem_match_criteria}



"""Explore example user views
"""
import json

from django.urls import reverse_lazy, reverse
from django.utils.decorators import method_decorator
from django.views.generic import View

from core_main_app.commons import exceptions as exceptions
from core_main_app.components.template import api as template_api
from core_main_app.components.template_version_manager import (
    api as template_version_manager_api,
)
from core_main_app.settings import DATA_SORTING_FIELDS
import core_main_app.utils.decorators as decorators
from core_main_app.utils.rendering import render

from core_explore_common_app.components.query import api as query_api
from core_explore_common_app.settings import DEFAULT_DATE_TOGGLE_VALUE
from core_explore_common_app.views.user.views import (
    ResultQueryRedirectView,
    ResultsView,
)
from core_explore_example_app.permissions import rights
from core_explore_example_app.components.explore_data_structure import (
    api as explore_data_structure_api,
)
from core_explore_example_app.components.persistent_query_example import (
    api as persistent_query_example_api,
)
from core_explore_example_app.components.persistent_query_example.models import (
    PersistentQueryExample,
)
from core_explore_example_app.components.saved_query import api as saved_query_api
from core_explore_example_app.settings import INSTALLED_APPS
from core_explore_example_app.utils.parser import render_form


class IndexView(View):
    """Index View"""

    api = template_version_manager_api
    get_redirect = "core_explore_example_app/user/index.html"
    select_object_redirect = "core_explore_example_select_fields"
    build_query_redirect = "core_explore_example_build_query"
    object_name = "template"

    @method_decorator(
        decorators.permission_required(
            content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
            permission=rights.EXPLORE_EXAMPLE_ACCESS,
            login_url=reverse_lazy("core_main_app_login"),
        )
    )
    def get(self, request, *args, **kwargs):
        """Page that allows to select a template to start exploring data

        Args:
            request:

        Returns:

        """
        assets = {"css": ["core_explore_example_app/user/css/style.css"]}

        global_active_template_list = self.get_global_active_list(request=request)
        user_active_template_list = self.get_user_active_list(request=request)

        context = {
            "global_objects": global_active_template_list,
            "user_objects": user_active_template_list,
            "object_name": self.object_name,
            "select_object_redirect": self.select_object_redirect,
            "build_query_redirect": self.build_query_redirect,
        }

        return render(request, self.get_redirect, assets=assets, context=context)

    def get_global_active_list(self, request):
        """Get all global version managers.

        Args:
            request:

        Returns:
            List of all global version managers

        """
        return self.api.get_active_global_version_manager(request=request)

    def get_user_active_list(self, request):
        """Get all active version managers with given user id.

        Args:
            request:

        Returns:
            List of all global version managers with given user.

        """
        return self.api.get_active_version_manager_by_user_id(request=request)


class SelectFieldsView(View):
    """Select Fields View"""

    build_query_url = "core_explore_example_build_query"
    load_form_url = "core_explore_example_load_form"
    generate_element_url = "core_explore_example_generate_element"
    remove_element_url = "core_explore_example_remove_element"
    generate_choice_url = "core_explore_example_generate_choice"

    @method_decorator(
        decorators.permission_required(
            content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
            permission=rights.EXPLORE_EXAMPLE_ACCESS,
            login_url=reverse_lazy("core_main_app_login"),
        )
    )
    def get(self, request, template_id, *args, **kwargs):
        """Loads view to customize exploration tree

        Args:
            request:
            template_id:

        Returns:

        """
        try:
            # Set the assets
            assets = {
                "js": [
                    {"path": "core_main_app/common/js/XMLTree.js", "is_raw": False},
                    {"path": "core_parser_app/js/autosave.js", "is_raw": False},
                    {
                        "path": "core_parser_app/js/autosave_checkbox.js",
                        "is_raw": False,
                    },
                    {"path": "core_parser_app/js/autosave.raw.js", "is_raw": True},
                    {"path": "core_parser_app/js/buttons.js", "is_raw": False},
                    {
                        "path": "core_explore_example_app/user/js/buttons.raw.js",
                        "is_raw": True,
                    },
                    {"path": "core_parser_app/js/modules.js", "is_raw": False},
                    {"path": "core_parser_app/js/choice.js", "is_raw": False},
                    {
                        "path": "core_explore_example_app/user/js/choice.raw.js",
                        "is_raw": True,
                    },
                    {
                        "path": "core_explore_example_app/user/js/select_fields.js",
                        "is_raw": False,
                    },
                    {
                        "path": "core_explore_example_app/user/js/select_fields.raw.js",
                        "is_raw": True,
                    },
                ],
                "css": [
                    "core_explore_example_app/user/css/xsd_form.css",
                    "core_explore_example_app/user/css/style.css",
                ],
            }

            template = template_api.get_by_id(template_id, request=request)
            # get data structure
            data_structure = (
                explore_data_structure_api.create_and_get_explore_data_structure(
                    template, request
                )
            )
            root_element = data_structure.data_structure_element_root

            # renders the form
            xsd_form = render_form(request, root_element)

            # Set the context
            context = {
                "template_id": template_id,
                "build_query_url": self.build_query_url,
                "load_form_url": self.load_form_url,
                "generate_element_url": self.generate_element_url,
                "remove_element_url": self.remove_element_url,
                "generate_choice_url": self.generate_choice_url,
                "data_structure_id": str(data_structure.id),
                "xsd_form": xsd_form,
            }

            return render(
                request,
                "core_explore_example_app/user/select_fields.html",
                assets=assets,
                context=context,
            )
        except Exception as exception:
            return render(
                request,
                "core_explore_example_app/user/errors.html",
                assets={},
                context={"errors": str(exception)},
            )


class BuildQueryView(View):
    """Build Query View"""

    build_query_url = "core_explore_example_build_query"
    get_query_url = "core_explore_example_get_query"
    save_query_url = "core_explore_example_save_query"
    results_url = "core_explore_example_results"
    select_fields_url = "core_explore_example_select_fields"
    object_name = "template"
    data_sources_selector_template = (
        "core_explore_common_app/user/selector/data_sources_selector" ".html"
    )
    query_builder_interface = (
        "core_explore_example_app/user/query_builder/initial_form.html"
    )

    @method_decorator(
        decorators.permission_required(
            content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
            permission=rights.EXPLORE_EXAMPLE_ACCESS,
            login_url=reverse_lazy("core_main_app_login"),
        )
    )
    def get(self, request, template_id, query_id=None):
        """Page that allows to build and submit queries

        Args:
            request:
            template_id:
            query_id:

        Returns:

        """
        try:
            template = template_api.get_by_id(template_id, request=request)
            if template is None:
                return render(
                    request,
                    "core_explore_example_app/user/errors.html",
                    assets={},
                    context={
                        "errors": "The selected {0} does not exist".format(
                            self.object_name
                        )
                    },
                )

            # Init variables
            saved_query_form = ""

            try:
                explore_data_structure = (
                    explore_data_structure_api.get_by_user_id_and_template_id(
                        str(request.user.id), template_id
                    )
                )
                # If custom fields form present, set it
                custom_form = explore_data_structure.selected_fields_html_tree
            except exceptions.DoesNotExist:
                custom_form = None

            # If new form
            if query_id is None:
                # empty session variables
                request.session["mapCriteriaExplore"] = dict()
                request.session["savedQueryFormExplore"] = ""
                # create new query object
                query = self._create_new_query(request, template)
            else:
                # if not a new form and a query form is present in session
                if "savedQueryFormExplore" in request.session:
                    saved_query_form = request.session["savedQueryFormExplore"]
                query = query_api.get_by_id(query_id, request.user)

            # Get saved queries of a user
            if "_auth_user_id" in request.session:
                user_id = request.session["_auth_user_id"]
                user_queries = saved_query_api.get_all_by_user_and_template(
                    user_id=user_id, template_id=template_id
                )
            else:
                user_queries = []

            assets = {"js": self._get_js(), "css": self._get_css()}

            context = {
                "queries": user_queries,
                "template_id": template_id,
                "description": self.get_description(),
                "title": self.get_title(),
                "data_sorting_fields": build_sorting_context_array(query),
                "default_data_sorting_fields": ",".join(DATA_SORTING_FIELDS),
                "custom_form": custom_form,
                "query_form": saved_query_form,
                "query_id": str(query.id),
                "build_query_url": self.build_query_url,
                "results_url": self.results_url,
                "get_query_url": self.get_query_url,
                "save_query_url": self.save_query_url,
                "select_fields_url": self.select_fields_url,
                "data_sources_selector_template": self.data_sources_selector_template,
                "query_builder_interface": self.query_builder_interface,
            }

            modals = [
                "core_explore_example_app/user/modals/custom_tree.html",
                "core_explore_example_app/user/modals/sub_elements_query_builder.html",
                "core_main_app/common/modals/error_page_modal.html",
                "core_explore_example_app/user/modals/delete_all_queries.html",
                "core_explore_example_app/user/modals/delete_query.html",
            ]

            return render(
                request,
                "core_explore_example_app/user/build_query.html",
                assets=assets,
                context=context,
                modals=modals,
            )
        except Exception as exception:
            return render(
                request,
                "core_explore_example_app/user/errors.html",
                assets={},
                context={"errors": str(exception)},
            )

    @staticmethod
    def _create_new_query(request, template):
        """Create a new query
        Args:
            request:
            template:

        """
        # from the template, we get the version manager
        template_version_manager = template.version_manager
        # from the version manager, we get all the version
        template_ids = template_api.get_all_accessible_by_id_list(
            template_version_manager.versions, request=request
        )
        # create query
        query = query_api.create_default_query(request, template_ids)
        return query

    @staticmethod
    def _get_js():
        return [
            {
                "path": "core_explore_example_app/user/js/build_query.js",
                "is_raw": False,
            },
            {
                "path": "core_explore_example_app/user/js/build_query.raw.js",
                "is_raw": True,
            },
            {"path": "core_parser_app/js/autosave.raw.js", "is_raw": True},
            {"path": "core_parser_app/js/choice.js", "is_raw": False},
            {
                "path": "core_main_app/common/js/modals/error_page_modal.js",
                "is_raw": True,
            },
        ]

    @staticmethod
    def _get_css():
        return [
            "core_explore_common_app/user/css/results.css",
            "core_explore_example_app/user/css/query_builder.css",
            "core_explore_example_app/user/css/xsd_form.css",
        ]

    @staticmethod
    def get_description():
        """get_description

        Returns
        """
        # FIXME should be in template
        return (
            "Click on a field of the Query Builder to add an element to your query. "
            "The elements selected in the previous step will appear and you will be "
            "able to insert them in the query builder. Click on plus/minus icons to "
            "add/remove criteria. You can save queries to build more complex queries "
            "and you will retrieve them on your next connection. When your query is "
            "done, please click on Submit Query to get XML documents that match the "
            "criteria."
        )

    @staticmethod
    def get_title():
        """get_description

        Returns
        """
        return "Query Builder"


class ResultQueryView(ResultsView):
    """Result Query View"""

    back_to_query_redirect = "core_explore_example_build_query"
    get_query_url = "core_explore_example_get_query"

    @method_decorator(
        decorators.permission_required(
            content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
            permission=rights.EXPLORE_EXAMPLE_ACCESS,
            login_url=reverse_lazy("core_main_app_login"),
        )
    )
    def get(self, request, template_id, query_id):
        """Query results view

        Args:
            request:
            template_id:
            query_id:

        Returns:

        """

        # get query
        query = query_api.get_by_id(query_id, request.user)

        context = {
            "template_id": template_id,
            "query_id": query_id,
            "exporter_app": False,
            "back_to_query_redirect": self.back_to_query_redirect,
            "get_query_url": self.get_query_url,
            "default_date_toggle_value": DEFAULT_DATE_TOGGLE_VALUE,
            "data_sorting_fields": super().build_sorting_context_array(query),
            "default_data_sorting_fields": ",".join(DATA_SORTING_FIELDS),
            "back_to_builder": True,
        }

        if "core_exporters_app" in INSTALLED_APPS:
            query = query_api.get_by_id(query_id, request.user)

            context["exporter_app"] = True
            context["templates_list"] = json.dumps(
                [str(template.id) for template in query.templates.all()]
            )

        return render(
            request,
            "core_explore_example_app/user/results.html",
            assets=self.assets,
            modals=self.modals,
            context=context,
        )

    def _load_assets(self):
        assets = super()._load_assets()
        extra_assets = {
            "js": [
                {
                    "path": "core_explore_example_app/user/js/refresh_sorting.raw.js",
                    "is_raw": True,
                },
                {
                    "path": "core_explore_example_app/user/js/refresh_sorting.js",
                    "is_raw": False,
                },
                {
                    "path": "core_explore_example_app/user/js/persistent_query.raw.js",
                    "is_raw": True,
                },
                {"path": "core_main_app/user/js/data/detail.js", "is_raw": False},
            ],
            "css": [],
        }

        assets["js"].extend(extra_assets["js"])
        assets["css"].extend(extra_assets["css"])

        return assets

    def _load_modals(self):
        """Return modals structure

        Returns:

        """
        return super()._load_modals()


class ResultQueryExampleRedirectView(ResultQueryRedirectView):
    """Result Query Example Redirect View"""

    model_name = PersistentQueryExample.__name__
    object_name = "persistent_query_example"
    redirect_url = "core_explore_example_results"

    @method_decorator(
        decorators.permission_required(
            content_type=rights.EXPLORE_EXAMPLE_CONTENT_TYPE,
            permission=rights.EXPLORE_EXAMPLE_ACCESS,
            login_url=reverse_lazy("core_main_app_login"),
        )
    )
    def get(self, request, *args, **kwargs):
        return super().get(self, request, *args, **kwargs)

    @staticmethod
    def _get_persistent_query_by_id(persistent_query_id, user):
        return persistent_query_example_api.get_by_id(persistent_query_id, user)

    @staticmethod
    def _get_persistent_query_by_name(persistent_query_name, user):
        return persistent_query_example_api.get_by_name(persistent_query_name, user)

    @staticmethod
    def get_url_path():
        return reverse(
            ResultQueryExampleRedirectView.redirect_url,
            kwargs={"template_id": "template_id", "query_id": "query_id"},
        ).split("results")[0]

    @staticmethod
    def _get_reversed_url(query):
        return reverse(
            ResultQueryExampleRedirectView.redirect_url,
            kwargs={"template_id": query.templates.all()[0].id, "query_id": query.id},
        )

    @staticmethod
    def _get_reversed_url_if_failed():
        return reverse("core_explore_example_index")


# FIXME duplicate code in core_explore_common_app
def build_sorting_context_array(query):
    """Get the query data-sources dans build the context sorting array for the JS

    Returns:

    """
    context_array = []
    for data_source in query.data_sources:
        context_array.append(data_source["order_by_field"])

    return ";".join(context_array)



"""Explore Example models
"""

from django.db import models

from core_main_app.permissions.utils import get_formatted_name
from core_explore_example_app.permissions import rights


class ExploreExample(models.Model):
    """Explore Example"""

    class Meta:
        """Meta"""

        verbose_name = "core_explore_example_app"
        default_permissions = ()
        permissions = (
            (
                rights.EXPLORE_EXAMPLE_ACCESS,
                get_formatted_name(rights.EXPLORE_EXAMPLE_ACCESS),
            ),
            (
                rights.EXPLORE_EXAMPLE_SAVE_QUERY,
                get_formatted_name(rights.EXPLORE_EXAMPLE_SAVE_QUERY),
            ),
            (
                rights.EXPLORE_EXAMPLE_DELETE_QUERY,
                get_formatted_name(rights.EXPLORE_EXAMPLE_DELETE_QUERY),
            ),
            (
                rights.EXPLORE_EXAMPLE_DATA_STRUCTURE_ACCESS,
                get_formatted_name(rights.EXPLORE_EXAMPLE_DATA_STRUCTURE_ACCESS),
            ),
        )



""" Fixture files for SavedQuery
"""

from core_main_app.components.template.models import Template
from core_main_app.utils.integration_tests.fixture_interface import FixtureInterface
from core_explore_example_app.components.saved_query.models import SavedQuery


class SavedQueryFixtures(FixtureInterface):
    """Saved Query fixtures"""

    data_1 = None
    data_collection = None

    def insert_data(self):
        """Insert a set of Data.

        Returns:

        """
        # Make a connexion with a mock database
        self.generate_data_collection()

    def generate_data_collection(self):
        """Generate a Data collection.

        Returns:

        """
        template = Template(filename="filename", content="<xml />", hash="hash")
        template.save()
        query_data_1 = (
            '{"$or": [{"list_content": {"$elemMatch": {"path": "/.*chemical-element-type/", "'
            'value": "Ac"}}}, {"list_content": {"$elemMatch": '
            '{"path": "/.*chemical-element-type.#text/", "value": "Ac"}}}]}'
        )
        self.data_1 = SavedQuery(
            user_id="1",
            template=template,
            query=query_data_1,
            displayed_query="Element is Ac",
        )
        self.data_1.save()
        self.data_collection = [self.data_1]



""" Saved Query Serializers
"""
from rest_framework.serializers import ModelSerializer

from core_explore_example_app.components.saved_query.models import SavedQuery


class SavedQuerySerializer(ModelSerializer):
    """Saved Query serializer"""

    class Meta:
        """Meta"""

        model = SavedQuery
        fields = "__all__"
        read_only_fields = ("user_id", "displayed_query")
